@@ -1,3 +1,27 @@
+/*******************************************************************************
+ lqt.h
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/
+
 #ifndef _LQT_H_
 #define _LQT_H_
 
@@ -6,6 +30,9 @@
 #include "lqt_codecinfo.h"
 #include "lqt_qtvr.h"
 
+#pragma GCC visibility push(default)
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -17,6 +44,17 @@ extern "C" {
   
 void *lqt_bufalloc(size_t size);
 
+
+  
+/** \ingroup log
+ *  \brief Set callback for global logging
+ *  \param cb Callback function
+ *  \param data Application supplied data
+ */
+
+void lqt_set_log_callback(lqt_log_callback_t cb, void * data);
+                           
+  
 /** \ingroup general
  *
  * \brief Return the raw filedescriptor associated with the file
@@ -38,10 +76,10 @@ int lqt_fileno(quicktime_t *file);
  *  \param value Parameter value.
  *
  *  For integer parameters, value must be of the type int*. For string parameters,
- *  use char*.
+ *  use char*.  For floating-point parameters, use float*.
  */
   
-void lqt_set_audio_parameter(quicktime_t *file,int track, char *key,void *value);
+void lqt_set_audio_parameter(quicktime_t *file,int track, const char *key,const void *value);
 
 /** \ingroup video
  *  \brief Set a codec parameter for a video track
@@ -51,10 +89,10 @@ void lqt_set_audio_parameter(quicktime_t *file,int track, char *key,void *value)
  *  \param value Parameter value.
  *
  *  For integer parameters, value must be of the type int*. For string parameters,
- *  use char*.
+ *  use char*.  For floating-point parameters, use float*.
  */
 
-void lqt_set_video_parameter(quicktime_t *file,int track, char *key,void *value);
+void lqt_set_video_parameter(quicktime_t *file,int track, const char *key,const void *value);
 
 /** \ingroup video_decode
  *  \brief Get the pixel aspect ratio of a video track
@@ -96,6 +134,22 @@ int lqt_set_pixel_aspect(quicktime_t *file, int track, int pixel_width,
 
 lqt_interlace_mode_t lqt_get_interlace_mode(quicktime_t * file, int track);
 
+/** \ingroup video_encode
+ *  \brief Set the interlace mode
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param mode The interlace mode.
+ *  \returns 1 if the call was successful, 0 if there is no such track
+ *
+ *  This will in most cases store the correct interlacing information
+ *  in the file (e.g. in the fiel atom). For tweaking the fiel atom
+ *  directly, advanced users might look at \ref lqt_set_fiel.
+ */
+
+int lqt_set_interlace_mode(quicktime_t * file, int track,
+                           lqt_interlace_mode_t mode);
+
+  
 /** \ingroup video
     \brief Convert an interlace mode to a human readable string
     \param mode An interlace mode
@@ -168,6 +222,389 @@ void lqt_set_channel_setup(quicktime_t * file, int track, lqt_channel_t * ch);
   
 const lqt_channel_t * lqt_get_channel_setup(quicktime_t * file, int track);
 
+
+/** \defgroup text Text
+    \brief Text related definitions and functions
+
+    Libquicktime supports reading and writing of text tracks from/to
+    Quicktime and mp4 files. Text tracks contain samples, which
+    are simple text strings. Libquicktime tries to handle character set issues
+    internally, all string you pass to/from libquicktime can assumed to be UTF-8.
+
+    A text track can either be used for subtitles (which is the default), or for
+    chapters. In the latter case, each text sample denotes the title of the chapter,
+    the corresponding timestamp is the start time of the chapter.
+
+    For subtitles, there is only a start time defined for each sample, not the duration,
+    after which the subtitle will be hidden. To make subtitles disappear at a specified
+    time, insert an empty subtitle with the right timestamp.
+
+    What's not supported are text attributes, font selection etc. Feel free to make a
+    proposal how to support these things.
+*/
+  
+/** \defgroup text_encode Writing text
+    \ingroup text
+    \brief Encode text
+
+    @{
+*/
+
+/** \brief Add a text track
+ *  \param file A quicktime handle
+ *  \param timescale The timescale, in which timestamps will be given.
+ */
+  
+int lqt_add_text_track(quicktime_t * file, int timescale);
+
+/** \brief Set the language for a text track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param language ISO-639 Language code
+ *
+ *  The language code is a 3-character code, English is "eng",
+ *  Japanese is "jpn".
+ */
+
+void lqt_set_text_language(quicktime_t * file, int track, const char * language);
+
+/** \brief Make a text track a chapter track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *
+ *  By default, text tracks are subtitles. By calling this function,
+ *  you tell libquicktime, that the text track should be a chapter track.
+ */
+
+
+  
+void lqt_set_chapter_track(quicktime_t * file, int track);
+
+/** \brief Set the text box of a text track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param top Top border
+ *  \param left Left border
+ *  \param bottom Bottom border
+ *  \param right Right border
+ */
+  
+void lqt_set_text_box(quicktime_t * file, int track,
+                      uint16_t top, uint16_t left,
+                      uint16_t bottom, uint16_t right);
+
+/** \brief Set the foreground color of a text track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param r Red
+ *  \param g Green
+ *  \param b Blue
+ *  \param a Alpha
+ *
+ *  Color values are between 0x0000 and 0xffff. The alpha value
+ *  is only used for mp4.
+ */
+  
+void lqt_set_text_fg_color(quicktime_t * file, int track,
+                           uint16_t r, uint16_t g,
+                           uint16_t b, uint16_t a);
+
+/** \brief Set the background color of a text track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param r Red
+ *  \param g Green
+ *  \param b Blue
+ *  \param a Alpha
+ *
+ *  Color values are between 0x0000 and 0xffff. The alpha value
+ *  is only used for mp4. For Quicktime, the text box is set to transparent
+ *  if the alpha value is below 0x8000.
+ */
+  
+void lqt_set_text_bg_color(quicktime_t * file, int track,
+                           uint16_t r, uint16_t g,
+                           uint16_t b, uint16_t a);
+
+  
+  
+  
+/** \brief Write a text sample
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param text A null-terminated UTF-8 string
+ *  \param duration The duration associated with this sample
+ *  \returns 0 if a the text sample could be written to the file, nonzero else
+ */
+  
+int lqt_write_text(quicktime_t * file, int track, const char * text, int64_t duration);
+  
+/**
+   @}
+*/
+  
+/** \defgroup text_decode Reading text
+    \ingroup text
+    \brief Decode text
+
+    @{
+
+*/
+
+/** \brief Get the number of text tracks
+ *  \param file A quicktime handle
+ *  \returns The number of text tracks found in the file
+ */
+  
+int lqt_text_tracks(quicktime_t * file);
+
+/** \brief Get the text language
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param language Returns ISO-639 Language code
+ *  \returns 1 on success, 0 on error.
+ *
+ *  The language code is a 3-character code, English is "eng",
+ *  Japanese is "jpn".
+ */
+
+int lqt_get_text_language(quicktime_t * file, int track, char * language);
+
+/** \brief Get the timescale for a text track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \returns The timescale of the track.
+ */
+
+int lqt_text_time_scale(quicktime_t * file, int track);
+
+/** \brief Set the text box of a text track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param top Returns the top border
+ *  \param left Returns the left border
+ *  \param bottom Returns the bottom border
+ *  \param right Returns the right border
+ */
+  
+void lqt_get_text_box(quicktime_t * file, int track,
+                      uint16_t * top, uint16_t * left,
+                      uint16_t * bottom, uint16_t * right);
+
+  
+/** \brief Read a text sample
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param text Address of a buffer
+ *  \param text_alloc Allocated bytes for this buffer (will be changed)
+ *  \param timestamp Returns the timestamp of the sample
+ *  \param duration Returns the duration  of the sample
+ *  \returns 1 if a sample was decoded, 0 if the track is finished
+ *
+ *  This funtion calls realloc() to make sure there is enough space for
+ *  for the text. It's a good idea to always pass the same buffer to this
+ *  function (set to NULL initially) and free it after the file is closed.
+ */
+  
+int lqt_read_text(quicktime_t * file, int track, char ** text, int * text_alloc,
+                  int64_t * timestamp, int64_t * duration);
+
+/** \brief Check if a track is a chapter track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \returns 1 if the text track is a chapter track, 0 else
+ */
+     
+  
+int lqt_is_chapter_track(quicktime_t * file, int track);
+
+/** \brief Get the total number of text samples
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \returns The number of samples
+ */
+
+int64_t lqt_text_samples(quicktime_t * file, int track);
+
+/** \brief Go to a specific sample
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param position The sample position (starting with 0)
+ */
+
+void lqt_set_text_position(quicktime_t * file, int track, int64_t position);
+
+/** \brief Go to a specific time
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param time Time
+ *
+ *  This wil reposition the text track such, that the next call
+ *  to \ref lqt_read_text will return a sample with at least the
+ *  timestamp you specified.
+ */
+
+void lqt_set_text_time(quicktime_t * file, int track, int64_t time);
+
+/** \brief Get the foreground color of a text track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param r Returns red
+ *  \param g Returns green
+ *  \param b Returns blue
+ *  \param a Returns alpha
+ *
+ *  Color values are between 0x0000 and 0xffff.
+ */
+  
+void lqt_get_text_fg_color(quicktime_t * file, int track,
+                           uint16_t * r, uint16_t * g,
+                           uint16_t * b, uint16_t * a);
+
+/** \brief Get the background color of a text track
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param r Returns red
+ *  \param g Returns green
+ *  \param b Returns blue
+ *  \param a Returns alpha
+ *
+ *  Color values are between 0x0000 and 0xffff.
+ */
+  
+void lqt_get_text_bg_color(quicktime_t * file, int track,
+                           uint16_t * r, uint16_t * g,
+                           uint16_t * b, uint16_t * a);
+
+/**
+   @}
+*/
+
+/** \defgroup timecodes Timecodes
+    \brief Timecode support
+
+    Timecodes are passed to/from libquicktime in the same format as they are stored in the
+    file: 32 bit unsigned integers. For the meaning of the bits, see the section "Timecode Sample Data"
+    in the Quicktime file format specification.
+    
+    @{
+
+*/
+  
+#define LQT_TIMECODE_DROP    0x0001 //!< Indicates whether the timecode is drop frame
+#define LQT_TIMECODE_24HMAX  0x0002 //!< Indicates whether the timecode wraps after 24 hours
+#define LQT_TIMECODE_NEG_OK  0x0004 //!< Indicates whether negative time values are allowed
+#define LQT_TIMECODE_COUNTER 0x0008 //!< Indicates whether the time value corresponds to a tape counter value
+
+/** \brief Attach a timecode track to a video track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param flags Zero or more of the LQT_TIMECODE_* flags
+ *  \param framerate The integer framerate
+ *
+ *  If the format (e.g. AVI) doesn't support timecode, this
+ *  function emits a warning.
+ *
+ *  Since 1.1.0
+ */
+
+void lqt_add_timecode_track(quicktime_t * file, int track,
+                            uint32_t flags, int framerate);
+
+/** \brief Write a timecode for the next video frame to be encoded
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *
+ *  Call this function @b before encoding the actual video frame.
+ *
+ *  Since 1.1.0
+ */
+  
+void lqt_write_timecode(quicktime_t * file, int track,
+                        uint32_t timecode);
+
+/** \brief Check, if a video track has timecodes
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param flags If non NULL returns zero or more of the LQT_TIMECODE_* flags
+ *  \param framerate If non NULL returns the integer framerate
+ *  \returns 1 if timecodes are available, 0 else
+ *
+ *  Since 1.1.0
+ */
+  
+int lqt_has_timecode_track(quicktime_t * file, int track,
+                           uint32_t * flags, int * framerate);
+
+/** \brief Read the timecode for the next frame to be decoded
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param timecode Returns the timecode if available
+ *  \returns 1 if a timecode is available for this frame, 0 else
+ *
+ *  For frames, which have no timecode attached, you can increment the
+ *  last timecode accordingly in your application.
+ *  Call this function @b before decoding the actual video frame.
+ *
+ *  Since 1.1.0
+ */
+
+  
+int lqt_read_timecode(quicktime_t * file, int track,
+                      uint32_t * timecode);
+
+/** \brief Get the tape name stored in a timecode track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \returns pointer to tape name (might be NULL if no tape name)
+ *
+ *  Returned pointer should remain valid as long as the file is open.
+ *  Do not free it.
+ *
+ *  Since 1.1.0
+ */
+  
+const char * lqt_get_timecode_tape_name(quicktime_t * file, int track);
+
+/** \brief Set the tapename for a timecode track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param tapename Tape name string
+ *
+ *  A copy is made of the string passed in
+ *
+ *  Since 1.1.0
+ */
+    
+void lqt_set_timecode_tape_name(quicktime_t * file, int track,
+				const char * tapename);
+
+/** \brief Get the enabled flag of a timecode track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \returns 1 if enabled, 0 if not enabled
+ *
+ *  Since 1.1.0
+ */
+
+int lqt_get_timecode_track_enabled(quicktime_t * file, int track);
+
+/** \brief Enable or disable a timecode track
+ *  \param file A quicktime handle
+ *  \param track Video track index (starting with 0)
+ *  \param enabled 0=>disable otherwise enable
+ *
+ *  Since 1.1.0
+ */
+
+void lqt_set_timecode_track_enabled(quicktime_t * file, int track,
+				    int enabled);
+
+/**
+   @}
+*/
+
+  
   
 /***********************************************
  * Advanced colormodel handling.
@@ -442,6 +879,18 @@ int lqt_add_audio_track(quicktime_t *file,
                         int channels, long sample_rate, int bits,
                         lqt_codec_info_t * codec_info);
 
+/** \ingroup audio_encode
+ *  \brief Set the audio language
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param language ISO-639 Language code
+ *
+ *  The language code is a 3-character code, English is "eng",
+ *  Japanese is "jpn".
+ */
+
+void lqt_set_audio_language(quicktime_t * file, int track, const char * language);
+  
 /** \ingroup video_encode
  *  \brief Add a video track for encoding
  *  \param file A quicktime handle
@@ -471,7 +920,7 @@ int lqt_add_video_track(quicktime_t *file,
  *  \param stats_file Statistics file
  *
  *  This is a purely optional function, which enables multipass encoding.
- *  Multipass transcoding is done by repeatedly opening a quicktime file,
+ *  Multipass encoding is done by repeatedly opening a quicktime file,
  *  encoding video and and closing it again. The stats_file parameter must
  *  always be the same for all passes.
  *
@@ -537,6 +986,7 @@ int lqt_read_video_frame(quicktime_t * file,
  *  \param row_pointers Frame (see \ref lqt_rows_alloc)
  *  \param track Track index (starting with 0)
  *  \param time Timestamp of the frame in timescale tics
+ *  \returns 0 if the frame was encoded, 1 else.
  *
  * Encode one video frame. The presentation timestamp is in
  * timescale tics with the timescale you passed to
@@ -548,6 +998,27 @@ int lqt_read_video_frame(quicktime_t * file,
 int lqt_encode_video(quicktime_t *file, 
                      unsigned char **row_pointers, 
                      int track, int64_t time);
+
+/** \ingroup video_encode
+ *  \brief Encode one video frame
+ *  \param file A quicktime handle
+ *  \param row_pointers Frame (see \ref lqt_rows_alloc)
+ *  \param track Track index (starting with 0)
+ *  \param time Timestamp of the frame in timescale tics
+ *  \param duration Duration of the frame
+ *  \returns 0 if the frame was encoded, 1 else.
+ *
+ *  This is the same as \ref lqt_encode_video except that you can
+ *  pass the duration along with the timestamp. This is really only
+ *  important for the last picture, since all other durations are
+ *  calculated from the timestamp differences.
+ *
+ *  Since 1.1.2
+ */
+  
+int lqt_encode_video_d(quicktime_t *file, 
+                       unsigned char **row_pointers, 
+                       int track, int64_t time, int duration);
   
 /** \ingroup video_decode
  *  \brief Get the duration of the NEXT frame to be decoded
@@ -763,12 +1234,27 @@ int lqt_decode_audio_raw(quicktime_t *file,
                          long samples,
                          int track);
 
+/** \ingroup audio_decode
+ *  \brief Get the audio language
+ *  \param file A quicktime handle
+ *  \param track Track index (starting with 0)
+ *  \param language Returns ISO-639 Language code
+ *  \returns 1 on success, 0 on error.
+ *
+ *  The language code is a 3-character code, English is "eng",
+ *  Japanese is "jpn".
+ */
+
+int lqt_get_audio_language(quicktime_t * file, int track, char * language);
+
+  
 /** \ingroup audio_encode
  * \brief Encode audio in the native sampleformat of the codec
  * \param file A quicktime handle
  * \param input An array of interleaved samples
  * \param samples Number of samples to encode
  * \param track Track index (starting with 0)
+ * \returns The number of encoded samples or 0 if an error occurred.
  *
  * This function bypasses all internal sampleformat conversion and allows
  * full resolution input for up to 32 bit integer and 32 bit float.
@@ -971,6 +1457,17 @@ lqt_file_type_t lqt_get_file_type(quicktime_t * file);
 
 quicktime_t * lqt_open_read(const char * filename);
 
+/** \ingroup general
+    \brief Open a file for reading
+    \param filename A path to a regular file
+    \param cb Log callback
+    \param log_data Data for log callback
+    \returns An initialized file handle or NULL if opening failed.
+    
+*/
+
+  quicktime_t * lqt_open_read_with_log(const char * filename, lqt_log_callback_t cb, void * log_data);
+  
 /** \ingroup general
     \brief Open a file for writing
     \param filename A path to a regular file
@@ -981,6 +1478,19 @@ quicktime_t * lqt_open_read(const char * filename);
 
 quicktime_t * lqt_open_write(const char * filename, lqt_file_type_t type);
 
+/** \ingroup general
+    \brief Open a file for writing
+    \param filename A path to a regular file
+    \param type The type of the file, you want to create
+    \param cb Log callback
+    \param log_data Data for log callback
+    \returns An initialized file handle or NULL if opening failed.
+    
+*/
+
+quicktime_t * lqt_open_write_with_log(const char * filename, lqt_file_type_t type,
+                                      lqt_log_callback_t cb, void * log_data);
+  
 /** \ingroup general
     \brief Set the segment size for ODML AVIs
     \param file A quicktime handle
@@ -1001,5 +1511,6 @@ void lqt_set_max_riff_size(quicktime_t * file, int size);
 }
 #endif /* __cplusplus */
 
+#pragma GCC visibility pop
   
 #endif
