@@ -1,4 +1,36 @@
-/* Taken from libsndfile */
+/*******************************************************************************
+ ulaw_tables.h
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+ 
+ libsndfile:
+ 
+ Copyright (C) 2002-2004 Erik de Castro Lopo
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+/*
+
+ From libsndfile.
+
+*/
 
 static short ulaw_decode [256] =
 {	-32124,	-31100,	-30076,	-29052,	-28028,	-27004,	-25980,	-24956,
