@@ -1,37 +1,43 @@
-/* 
- * audio.c (C) Justin Schoeman 2002 (justin@suntiger.ee.up.ac.za)
- * Based entirely upon qtpng.c from libquicktime 
- * (http://libquicktime.sf.net).
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA    
- */
+/*******************************************************************************
+ audio.c
 
-#include <string.h>
-#include "config.h"
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
 
-#include "funcprotos.h"
-#include <quicktime/quicktime.h>
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
+#include "ffmpeg.h"
 #define LQT_LIBQUICKTIME
 #include <quicktime/lqt_codecapi.h>
+#include <stdlib.h>
+#include <string.h>
 
-#include "funcprotos.h"
-#include <quicktime/colormodels.h>
-
-#include "ffmpeg.h"
+#define LOG_DOMAIN "ffmpeg_audio"
 
+/* Different decoding functions */
 
+#if LIBAVCODEC_BUILD >= ((51<<16)+(29<<8)+0)
+#define DECODE_FUNC avcodec_decode_audio2
+#else
+#define DECODE_FUNC avcodec_decode_audio
+#endif
 
 /* The following code was ported from gmerlin_avdecoder (http://gmerlin.sourceforge.net) */
 
@@ -84,26 +90,25 @@ static int mpeg_samplerates[3][3] = {
 /* Header detection stolen from the mpg123 plugin of xmms */
 
 static int header_check(uint32_t head)
-{
-        if ((head & 0xffe00000) != 0xffe00000)
-                return 0;
-        if (!((head >> 17) & 3))
-                return 0;
-        if (((head >> 12) & 0xf) == 0xf)
-                return 0;
-        if (!((head >> 12) & 0xf))
-                return 0;
-        if (((head >> 10) & 0x3) == 0x3)
-                return 0;
-        if (((head >> 19) & 1) == 1 &&
-            ((head >> 17) & 3) == 3 &&
-            ((head >> 16) & 1) == 1)
-                return 0;
-        if ((head & 0xffff0000) == 0xfffe0000)
-                return 0;
-        //        fprintf(stderr, "Head check ok %08x\n", head);
-        return 1;
-}
+  {
+  if ((head & 0xffe00000) != 0xffe00000)
+    return 0;
+  if (!((head >> 17) & 3))
+    return 0;
+  if (((head >> 12) & 0xf) == 0xf)
+    return 0;
+  if (!((head >> 12) & 0xf))
+    return 0;
+  if (((head >> 10) & 0x3) == 0x3)
+    return 0;
+  if (((head >> 19) & 1) == 1 &&
+      ((head >> 17) & 3) == 3 &&
+      ((head >> 16) & 1) == 1)
+    return 0;
+  if ((head & 0xffff0000) == 0xfffe0000)
+    return 0;
+  return 1;
+  }
 
 typedef enum
   {
@@ -295,10 +300,11 @@ typedef struct
   int have_mpeg_header;
 
   uint8_t * extradata;
-    
+#if LIBAVCODEC_BUILD >= ((52<<16)+(26<<8)+0)
+  AVPacket pkt;
+#endif
   } quicktime_ffmpeg_audio_codec_t;
 
-
 static int lqt_ffmpeg_delete_audio(quicktime_audio_map_t *vtrack)
   {
   quicktime_ffmpeg_audio_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
@@ -307,8 +313,7 @@ static int lqt_ffmpeg_delete_audio(quicktime_audio_map_t *vtrack)
     {
     if(codec->initialized)
       avcodec_close(codec->avctx);
-    else
-      free(codec->avctx);
+    av_free(codec->avctx);
     }
   if(codec->sample_buffer) free(codec->sample_buffer);
   if(codec->chunk_buffer)  free(codec->chunk_buffer);
@@ -319,8 +324,8 @@ static int lqt_ffmpeg_delete_audio(quicktime_audio_map_t *vtrack)
 
 static int set_parameter(quicktime_t *file, 
                   int track, 
-                  char *key, 
-                  void *value)
+                  const char *key, 
+                  const void *value)
   {
   quicktime_ffmpeg_audio_codec_t *codec = ((quicktime_codec_t*)file->atracks[track].codec)->priv;
   lqt_ffmpeg_set_parameter(codec->avctx, key, value);
@@ -335,51 +340,69 @@ static int decode_chunk_vbr(quicktime_t * file, int track)
   int chunk_packets, i, num_samples, bytes_decoded;
   int packet_size, packet_samples;
   int frame_bytes;
+  int new_samples;
   quicktime_audio_map_t *track_map = &(file->atracks[track]);
   quicktime_ffmpeg_audio_codec_t *codec = ((quicktime_codec_t*)track_map->codec)->priv;
 
-  chunk_packets = lqt_audio_num_vbr_packets(file, track, track_map->current_chunk, &num_samples);
+  chunk_packets = lqt_audio_num_vbr_packets(file, track, track_map->cur_chunk, &num_samples);
 
   if(!chunk_packets)
     return 0;
+
+  new_samples = num_samples + AVCODEC_MAX_AUDIO_FRAME_SIZE / (2 * track_map->channels);
   
-  if(codec->sample_buffer_alloc < codec->sample_buffer_end - codec->sample_buffer_start + num_samples)
+  if(codec->sample_buffer_alloc <
+     codec->sample_buffer_end - codec->sample_buffer_start + new_samples)
     {
     
-    codec->sample_buffer_alloc = codec->sample_buffer_end - codec->sample_buffer_start + num_samples;
-    //    fprintf(stderr, "codec->sample_buffer_alloc: %d\n", codec->sample_buffer_alloc);
+    codec->sample_buffer_alloc = codec->sample_buffer_end - codec->sample_buffer_start + new_samples;
     codec->sample_buffer = realloc(codec->sample_buffer, 2 * codec->sample_buffer_alloc *
                                    track_map->channels);
     }
 
   for(i = 0; i < chunk_packets; i++)
     {
-    packet_size = lqt_audio_read_vbr_packet(file, track, track_map->current_chunk, i,
+    packet_size = lqt_audio_read_vbr_packet(file, track, track_map->cur_chunk, i,
                                             &(codec->chunk_buffer), &(codec->chunk_buffer_alloc),
                                             &packet_samples);
     if(!packet_size)
       return 0;
+
+#if LIBAVCODEC_BUILD >= 3349760
+    bytes_decoded = codec->sample_buffer_alloc -
+      (codec->sample_buffer_end - codec->sample_buffer_start);
+    bytes_decoded *= 2 * track_map->channels;
+#else
+    bytes_decoded = 0;
+#endif
+
+#if LIBAVCODEC_BUILD >= ((52<<16)+(26<<8)+0)
+    codec->pkt.data = codec->chunk_buffer;
+    codec->pkt.size = packet_size + FF_INPUT_BUFFER_PADDING_SIZE;
     
+    frame_bytes = avcodec_decode_audio3(codec->avctx,
+                                        &(codec->sample_buffer[track_map->channels *
+                                                               (codec->sample_buffer_end -
+                                                                codec->sample_buffer_start)]),
+                                        &bytes_decoded,
+                                        &codec->pkt);
+#else
     frame_bytes =
-      avcodec_decode_audio(codec->avctx,
-                           &(codec->sample_buffer[track_map->channels *
-                                                  (codec->sample_buffer_end - codec->sample_buffer_start)]),
-                           &bytes_decoded,
-                           codec->chunk_buffer,
-                           packet_size + FF_INPUT_BUFFER_PADDING_SIZE);
+      DECODE_FUNC(codec->avctx,
+                  &(codec->sample_buffer[track_map->channels *
+                                         (codec->sample_buffer_end - codec->sample_buffer_start)]),
+                  &bytes_decoded,
+                  codec->chunk_buffer,
+                  packet_size + FF_INPUT_BUFFER_PADDING_SIZE);
+#endif
     if(frame_bytes < 0)
       {
-      fprintf(stderr, "avcodec_decode_audio error\n");
+      lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "avcodec_decode_audio error");
       break;
       }
-#if 0
-    fprintf(stderr, "decode_chunk_vbr: Samples decoded: %d, Bytes used: %d\n",
-            bytes_decoded / (track_map->channels * 2),
-            frame_bytes);
-#endif
     codec->sample_buffer_end += (bytes_decoded / (track_map->channels * 2));
     }
-  track_map->current_chunk++;
+  track_map->cur_chunk++;
   return num_samples;
   }
 
@@ -392,6 +415,7 @@ static int decode_chunk(quicktime_t * file, int track)
     
   int frame_bytes;
   int num_samples;
+  int new_samples;
   int samples_decoded = 0;
   int bytes_decoded;
   int bytes_used, bytes_skipped;
@@ -402,17 +426,14 @@ static int decode_chunk(quicktime_t * file, int track)
   /* Read chunk */
   
   chunk_size = lqt_append_audio_chunk(file,
-                                      track, track_map->current_chunk,
+                                      track, track_map->cur_chunk,
                                       &(codec->chunk_buffer),
                                       &(codec->chunk_buffer_alloc),
                                       codec->bytes_in_chunk_buffer);
 
-  //  fprintf(stderr, "Got chunk:\n");
-  //  lqt_hexdump(codec->chunk_buffer, 16, 16);
   
   if(!chunk_size)
     {
-    //    fprintf(stderr, "audio_ffmpeg: EOF 1 (%d bytes left)\n", codec->bytes_in_chunk_buffer);
     /* If the codec is mp3, make sure to decode the very last frame */
 
     if((codec->avctx->codec_id == CODEC_ID_MP3) &&
@@ -420,12 +441,12 @@ static int decode_chunk(quicktime_t * file, int track)
       {
       if(!decode_header(&mph, codec->chunk_buffer, (const mpeg_header*)0))
         {
-        fprintf(stderr, "Decode header failed\n");
+        lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "Decode header failed");
         return 0;
         }
       if(mph.frame_bytes <= codec->bytes_in_chunk_buffer)
         {
-        fprintf(stderr, "Huh, frame not decoded?\n");
+        lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "Huh, frame not decoded?");
         return 0;
         }
 
@@ -444,17 +465,14 @@ static int decode_chunk(quicktime_t * file, int track)
     }
   else
     {
-    //  fprintf(stderr, "decode chunk, bytes from last chunk: %d\n", codec->bytes_in_chunk_buffer);
-    num_samples = quicktime_chunk_samples(track_map->track, track_map->current_chunk);
-    track_map->current_chunk++;
+    num_samples = quicktime_chunk_samples(track_map->track, track_map->cur_chunk);
+    track_map->cur_chunk++;
     codec->bytes_in_chunk_buffer += chunk_size;
     }
   
-  //  fprintf(stderr, "chunk samples: %d\n", num_samples);
 
   if(!num_samples)
     {
-    //    fprintf(stderr, "audio_ffmpeg: EOF\n");
     return 0;
     }
   /*
@@ -463,42 +481,27 @@ static int decode_chunk(quicktime_t * file, int track)
    */
 
   num_samples += 8192;
-
+  new_samples = num_samples + AVCODEC_MAX_AUDIO_FRAME_SIZE / (2 * track_map->channels);
+  
   /* Reallocate sample buffer */
   
-  if(codec->sample_buffer_alloc < codec->sample_buffer_end - codec->sample_buffer_start + num_samples)
+  if(codec->sample_buffer_alloc < codec->sample_buffer_end - codec->sample_buffer_start + new_samples)
     {
     
-    codec->sample_buffer_alloc = codec->sample_buffer_end - codec->sample_buffer_start + num_samples;
-    //    fprintf(stderr, "codec->sample_buffer_alloc: %d\n", codec->sample_buffer_alloc);
+    codec->sample_buffer_alloc = codec->sample_buffer_end - codec->sample_buffer_start + new_samples;
     codec->sample_buffer = realloc(codec->sample_buffer, 2 * codec->sample_buffer_alloc *
                                    track_map->channels);
     }
   
-  
   /* Decode this */
 
   bytes_used = 0;
   while(1)
     {
-#if 0
-    fprintf(stderr, "Avcodec decode audio %d, header: ",
-            codec->bytes_in_chunk_buffer);
-    fprintf(stderr, "%02x %02x %02x %02x\n",
-            codec->chunk_buffer[bytes_used],
-            codec->chunk_buffer[bytes_used+1],
-            codec->chunk_buffer[bytes_used+2],
-            codec->chunk_buffer[bytes_used+3]);
-#endif
 
         
     /* BIG NOTE: We pass extra FF_INPUT_BUFFER_PADDING_SIZE for the buffer size
        because we know, that lqt_read_audio_chunk allocates 16 extra bytes for us */
-#if 0
-    fprintf(stderr, "decode_chunk: Sample buffer: %d, chunk_buffer: %d\n",
-            (int)(codec->sample_buffer_end - codec->sample_buffer_start),
-            codec->bytes_in_chunk_buffer);
-#endif
     
     /* Some really broken mp3 files have the header bytes split across 2 chunks */
 
@@ -506,7 +509,6 @@ static int decode_chunk(quicktime_t * file, int track)
       {
       if(codec->bytes_in_chunk_buffer < 4)
         {
-        //        fprintf(stderr, "decode_chunk: Incomplete frame %d\n", codec->bytes_in_chunk_buffer);
         
         if(codec->bytes_in_chunk_buffer > 0)
           memmove(codec->chunk_buffer,
@@ -534,7 +536,6 @@ static int decode_chunk(quicktime_t * file, int track)
         bytes_skipped++;
         if(codec->bytes_in_chunk_buffer <= 4)
           {
-          //          fprintf(stderr, "decode_chunk: Incomplete frame %d\n", codec->bytes_in_chunk_buffer);
 
           if(codec->bytes_in_chunk_buffer > 0)
             memmove(codec->chunk_buffer,
@@ -542,14 +543,8 @@ static int decode_chunk(quicktime_t * file, int track)
           return 1;
           }
         }
-#if 0
-      if(bytes_skipped)
-        fprintf(stderr, "Skipped %d bytes, bytes_in_chunk_buffer: %d\n",
-                bytes_skipped, codec->bytes_in_chunk_buffer);
-#endif 
       if(codec->bytes_in_chunk_buffer < mph.frame_bytes)
         {
-        //        fprintf(stderr, "decode_chunk: Incomplete frame %d\n", codec->bytes_in_chunk_buffer);
         
         if(codec->bytes_in_chunk_buffer > 0)
           memmove(codec->chunk_buffer,
@@ -566,24 +561,39 @@ static int decode_chunk(quicktime_t * file, int track)
      *  frame_size_ptr is zero. Otherwise, it is the decompressed frame
      *  size in BYTES.
      */
+
+#if LIBAVCODEC_BUILD >= 3349760
+    bytes_decoded = codec->sample_buffer_alloc -
+      (codec->sample_buffer_end - codec->sample_buffer_start);
+    bytes_decoded *= 2 * track_map->channels;
+#else
+    bytes_decoded = 0;
+#endif
+
+#if LIBAVCODEC_BUILD >= ((52<<16)+(26<<8)+0)
+    codec->pkt.data = &(codec->chunk_buffer[bytes_used]);
+    codec->pkt.size = codec->bytes_in_chunk_buffer + FF_INPUT_BUFFER_PADDING_SIZE;
     
     frame_bytes =
-      avcodec_decode_audio(codec->avctx,
-                           &(codec->sample_buffer[track_map->channels *
-                                                  (codec->sample_buffer_end - codec->sample_buffer_start)]),
-                           &bytes_decoded,
-                           &(codec->chunk_buffer[bytes_used]),
-                           codec->bytes_in_chunk_buffer + FF_INPUT_BUFFER_PADDING_SIZE);
+      avcodec_decode_audio3(codec->avctx,
+                            &(codec->sample_buffer[track_map->channels *
+                                                   (codec->sample_buffer_end - codec->sample_buffer_start)]),
+                            &bytes_decoded,
+                            &codec->pkt);
+#else
+    frame_bytes =
+      DECODE_FUNC(codec->avctx,
+                  &(codec->sample_buffer[track_map->channels *
+                                         (codec->sample_buffer_end - codec->sample_buffer_start)]),
+                  &bytes_decoded,
+                  &(codec->chunk_buffer[bytes_used]),
+                  codec->bytes_in_chunk_buffer + FF_INPUT_BUFFER_PADDING_SIZE);
+#endif
     if(frame_bytes < 0)
       {
-      fprintf(stderr, "avcodec_decode_audio error\n");
+      lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "avcodec_decode_audio error");
       break;
       }
-#if 0
-    fprintf(stderr, "decode_chunk: Samples decoded: %d, Bytes used: %d, chunk_size: %lld, chunk buffer: %d\n",
-            bytes_decoded / (track_map->channels * 2),
-            frame_bytes, chunk_size, codec->bytes_in_chunk_buffer);
-#endif
 
     bytes_used                   += frame_bytes;
     codec->bytes_in_chunk_buffer -= frame_bytes;
@@ -609,10 +619,6 @@ static int decode_chunk(quicktime_t * file, int track)
       else
         {
         /* Incomplete frame, save the data for later use and exit here */
-#if 0
-        fprintf(stderr, "decode_chunk: Incomplete frame %d %d\n", bytes_decoded,
-                codec->bytes_in_chunk_buffer);
-#endif   
         if(codec->bytes_in_chunk_buffer > 0)
           memmove(codec->chunk_buffer,
                   codec->chunk_buffer + bytes_used, codec->bytes_in_chunk_buffer);
@@ -636,7 +642,7 @@ static int decode_chunk(quicktime_t * file, int track)
     codec->sample_buffer_end += (bytes_decoded / (track_map->channels * 2));
 
     if((int)(codec->sample_buffer_end - codec->sample_buffer_start) > codec->sample_buffer_alloc)
-      fprintf(stderr, "BUUUUG, buffer overflow, %d %d\n",
+      lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "BUUUUG, buffer overflow, %d %d",
               (int)(codec->sample_buffer_end - codec->sample_buffer_start),
               codec->sample_buffer_alloc);
     
@@ -644,11 +650,6 @@ static int decode_chunk(quicktime_t * file, int track)
       break;
 
     }
-#if 0
-  fprintf(stderr, "Last samples decoded: %d, Bytes used: %d, samples decoded %d, chunk_size: %lld\n",
-          bytes_decoded / (track_map->channels * 2),
-          bytes_used, samples_decoded, chunk_size);
-#endif
   //  track_map->current_chunk++;
   return samples_decoded;
   }
@@ -669,7 +670,6 @@ static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long sample
   int samples_to_skip;
   int samples_to_move;
 
-  //  fprintf(stderr, "ffmpeg decode audio %lld %d\n", track_map->current_position, samples);
 
   if(!output) /* Global initialization */
     {
@@ -693,9 +693,11 @@ static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long sample
     
     //  priv->ctx->block_align     = s->data.audio.block_align;
     //  priv->ctx->bit_rate        = s->codec_bitrate;
-
+#if LIBAVCODEC_VERSION_INT < ((52<<16)+(0<<8)+0)
     codec->avctx->bits_per_sample = quicktime_audio_bits(file, track);
-
+#else
+    codec->avctx->bits_per_coded_sample = quicktime_audio_bits(file, track);
+#endif
     /* Some codecs need extra stuff */
 
     if(codec->decoder->id == CODEC_ID_ALAC)
@@ -727,11 +729,12 @@ static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long sample
 
       }
 
+    
     //    memcpy(&(codec->com.ffcodec_enc), &(codec->com.params), sizeof(AVCodecContext));
     
     if(avcodec_open(codec->avctx, codec->decoder) != 0)
       {
-      fprintf(stderr, "Avcodec open failed\n");
+      lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "Avcodec open failed");
       return 0;
       }
     
@@ -749,20 +752,14 @@ static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long sample
       {
       if(lqt_audio_is_vbr(file, track))
         lqt_chunk_of_sample_vbr(&chunk_sample,
-                                &(track_map->current_chunk),
+                                &(track_map->cur_chunk),
                                 track_map->track,
                                 track_map->current_position);
       else
         quicktime_chunk_of_sample(&chunk_sample,
-                                  &(track_map->current_chunk),
+                                  &(track_map->cur_chunk),
                                   track_map->track,
                                   track_map->current_position);
-#if 0
-      fprintf(stderr, "Seek: last_pos: %ld, pos: %lld, chunk: %lld, chunk_sample: %lld\n",
-              track_map->last_position,
-              track_map->current_position,
-              track_map->current_chunk, chunk_sample);
-#endif       
       codec->sample_buffer_start = chunk_sample;
       codec->sample_buffer_end   = chunk_sample;
       codec->bytes_in_chunk_buffer = 0;
@@ -779,10 +776,6 @@ static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long sample
   if(track_map->current_position > codec->sample_buffer_start)
     {
     samples_to_skip = track_map->current_position - codec->sample_buffer_start;
-#if 0
-    fprintf(stderr, "** Flush samples_to_skip: %d, %d\n", samples_to_skip,
-            (int)(codec->sample_buffer_end - codec->sample_buffer_start));
-#endif
     if(samples_to_skip > (int)(codec->sample_buffer_end - codec->sample_buffer_start))
       samples_to_skip = (int)(codec->sample_buffer_end - codec->sample_buffer_start);
     
@@ -790,15 +783,12 @@ static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long sample
       {
       samples_to_move = codec->sample_buffer_end - track_map->current_position;
 
-      //      fprintf(stderr, "Memmove...");
       memmove(codec->sample_buffer,
               &(codec->sample_buffer[samples_to_skip * channels]),
               samples_to_move * channels * sizeof(int16_t));
-      //      fprintf(stderr, "done\n");
       }
     codec->sample_buffer_start += samples_to_skip;
     
-    //    fprintf(stderr, " %d\n", (int)(codec->sample_buffer_end - codec->sample_buffer_start));
 
     }
   samples_to_skip = track_map->current_position - codec->sample_buffer_start;
@@ -806,11 +796,6 @@ static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long sample
   /* Read new chunks until we have enough samples */
   while(codec->sample_buffer_end - codec->sample_buffer_start < samples + samples_to_skip)
     {
-#if 0
-    fprintf(stderr, "Samples: %lld -> %lld %lld, %d\n",
-            codec->sample_buffer_start, codec->sample_buffer_end,
-            codec->sample_buffer_end - codec->sample_buffer_start, samples);
-#endif
     
     //    if(track_map->current_chunk >= track_map->track->mdia.minf.stbl.stco.total_entries)
     //      return 0;
@@ -828,10 +813,15 @@ static int lqt_ffmpeg_decode_audio(quicktime_t *file, void * output, long sample
     }
   samples_decoded = codec->sample_buffer_end - codec->sample_buffer_start - samples_to_skip;
 
+  
+  if(samples_decoded <= 0)
+    {
+    track_map->last_position = track_map->current_position;
+    return 0;
+    }
   if(samples_decoded > samples)
     samples_decoded = samples;
   
-  //  fprintf(stderr, "Samples_decoded: %d\n", samples_decoded);
   /* Deinterleave into the buffer */
   
   
@@ -872,7 +862,7 @@ static int lqt_ffmpeg_encode_audio(quicktime_t *file, void * input,
 
     if(avcodec_open(codec->avctx, codec->encoder) != 0)
       {
-      fprintf(stderr, "Avcodec open failed\n");
+      lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "Avcodec open failed");
       return -1;
       }
     
@@ -913,11 +903,8 @@ static int lqt_ffmpeg_encode_audio(quicktime_t *file, void * input,
   
   /* Encode */
   
-  //  fprintf(stderr, "codec->samples_in_buffer: %d, codec->avctx->frame_size %d\n",
-  //          codec->samples_in_buffer, codec->avctx->frame_size);
   while(codec->samples_in_buffer >= codec->avctx->frame_size)
     {
-    //    fprintf(stderr, "avcodec_encode_audio %d...", samples_done);
     
     frame_bytes = avcodec_encode_audio(codec->avctx, codec->chunk_buffer,
                                        codec->chunk_buffer_alloc,
@@ -932,7 +919,6 @@ static int lqt_ffmpeg_encode_audio(quicktime_t *file, void * input,
 #endif
         samples_encoded = codec->avctx->frame_size;
 
-      //      fprintf(stderr, "Done %d->%d\n", samples_encoded, frame_bytes);
 
       samples_done              += samples_encoded;
       codec->samples_in_buffer  -= samples_encoded;
@@ -940,10 +926,10 @@ static int lqt_ffmpeg_encode_audio(quicktime_t *file, void * input,
       result = !quicktime_write_data(file, codec->chunk_buffer, frame_bytes);
       quicktime_write_chunk_footer(file,
                                    trak, 
-                                   file->atracks[track].current_chunk,
+                                   file->atracks[track].cur_chunk,
                                    &chunk_atom, 
                                    samples_encoded);
-      file->atracks[track].current_chunk++;
+      file->atracks[track].cur_chunk++;
       }
     }
   if(codec->samples_in_buffer && samples_done)
@@ -958,7 +944,6 @@ void quicktime_init_audio_codec_ffmpeg(quicktime_audio_map_t *atrack, AVCodec *e
 	quicktime_ffmpeg_audio_codec_t *codec;
 
 	avcodec_init();
-        fprintf(stderr, "quicktime_init_audio_codec_ffmpeg 1\n");
 	codec = calloc(1, sizeof(quicktime_ffmpeg_audio_codec_t));
 	if(!codec)
           return;
