@@ -1,30 +1,43 @@
-/* 
- * video.c (C) Justin Schoeman 2002 (justin@suntiger.ee.up.ac.za)
- * Based entirely upon qtpng.c from libquicktime 
- * (http://libquicktime.sf.net).
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- */
+/*******************************************************************************
+ video.c
 
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
 
-#include "config.h"
-#include "funcprotos.h"
-#include <quicktime/colormodels.h>
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
 #include "ffmpeg.h"
-
+#include <quicktime/colormodels.h>
+#include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 
+#define LOG_DOMAIN "ffmpeg_video"
+
+#ifdef HAVE_LIBSWSCALE
+#include SWSCALE_HEADER
+#endif
+
+// Enable interlaced encoding (experimental)
+// #define DO_INTERLACE
+
 typedef struct
   {
   AVCodecContext * avctx;
@@ -43,6 +56,9 @@ typedef struct
   /* Colormodel */
 
   int do_imgconvert;
+#ifdef HAVE_LIBSWSCALE
+  struct SwsContext *swsContext;
+#endif
     
   //  unsigned char ** tmp_buffer;
   unsigned char ** row_pointers;
@@ -58,8 +74,6 @@ typedef struct
 
   int have_frame;
 
-  int encode_colormodel;
-
   int write_global_header;
   int global_header_written;
 
@@ -70,7 +84,10 @@ typedef struct
   int pass;
   char * stats_filename;
   FILE * stats_file;
-  
+
+#if LIBAVCODEC_BUILD >= ((52<<16)+(26<<8)+0)
+  AVPacket pkt;
+#endif
   } quicktime_ffmpeg_video_codec_t;
 
 /* ffmpeg <-> libquicktime colormodels */
@@ -86,7 +103,11 @@ static struct
 colormodels[] =
   {
     { PIX_FMT_YUV420P,  BC_YUV420P,  1 },  ///< Planar YUV 4:2:0 (1 Cr & Cb sample per 2x2 Y samples)
+#if LIBAVUTIL_VERSION_INT < (50<<16)
     { PIX_FMT_YUV422,   BC_YUV422,   1 },
+#else
+    { PIX_FMT_YUYV422,   BC_YUV422,  1 },
+#endif
     { PIX_FMT_RGB24,    BC_RGB888,   1 },  ///< Packed pixel, 3 bytes per pixel, RGBRGB...
     { PIX_FMT_BGR24,    BC_BGR888,   1 },  ///< Packed pixel, 3 bytes per pixel, BGRBGR...
     { PIX_FMT_YUV422P,  BC_YUV422P,  1 },  ///< Planar YUV 4:2:2 (1 Cr & Cb sample per 2x1 Y samples)
@@ -96,7 +117,11 @@ colormodels[] =
     { PIX_FMT_YUVJ420P, BC_YUVJ420P, 1 }, ///< Planar YUV 4:2:0 full scale (jpeg)
     { PIX_FMT_YUVJ422P, BC_YUVJ422P, 1 }, ///< Planar YUV 4:2:2 full scale (jpeg)
     { PIX_FMT_YUVJ444P, BC_YUVJ444P, 1 }, ///< Planar YUV 4:4:4 full scale (jpeg)
+#if LIBAVUTIL_VERSION_INT < (50<<16)
     { PIX_FMT_RGBA32,   BC_RGBA8888, 0 },  ///< Packed pixel, 4 bytes per pixel, BGRABGRA...
+#else
+    { PIX_FMT_RGB32,   BC_RGBA8888, 0 },  ///< Packed pixel, 4 bytes per pixel, BGRABGRA...
+#endif
     { PIX_FMT_RGB555,   BC_RGB888,   0 },  ///< always stored in cpu endianness, most significant bit to 1
     { PIX_FMT_GRAY8,    BC_RGB888,   0 },
     { PIX_FMT_MONOWHITE, BC_RGB888,  0 },///< 0 is white
@@ -106,38 +131,44 @@ colormodels[] =
   };
 
 static int lqt_ffmpeg_delete_video(quicktime_video_map_t *vtrack)
-{
-	quicktime_ffmpeg_video_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
+  {
+  quicktime_ffmpeg_video_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
           
-        if(codec->extradata)
-          free(codec->extradata);
+  if(codec->extradata)
+    free(codec->extradata);
 
-        if(codec->stats_filename)
-          free(codec->stats_filename);
+  if(codec->stats_filename)
+    free(codec->stats_filename);
         
-        if(codec->stats_file)
-          fclose(codec->stats_file);
+  if(codec->stats_file)
+    fclose(codec->stats_file);
         
-	if(codec->initialized)
-          {
-          if(codec->avctx->stats_in)
-            free(codec->avctx->stats_in);
-          avcodec_close(codec->avctx);
-          }
-        else
-          free(codec->avctx);
+  if(codec->initialized)
+    {
+    if(codec->avctx->stats_in)
+      av_free(codec->avctx->stats_in);
+    avcodec_close(codec->avctx);
+    }
+  av_free(codec->avctx);
         
-        if(codec->frame_buffer) free(codec->frame_buffer);
-	if(codec->buffer) free(codec->buffer);
-        if(codec->row_pointers) free(codec->row_pointers);
+  if(codec->frame_buffer) free(codec->frame_buffer);
+  if(codec->buffer) free(codec->buffer);
+  if(codec->row_pointers) free(codec->row_pointers);
+
+  if(codec->frame) av_free(codec->frame);
+
+#ifdef HAVE_LIBSWSCALE
+  if(codec->swsContext)
+    sws_freeContext(codec->swsContext);
+#endif
 
-        if(codec->frame) free(codec->frame);
         
-	free(codec);
+  free(codec);
 	
-	return 0;
-}
+  return 0;
+  }
 
+#ifndef HAVE_LIBSWSCALE
 static void fill_avpicture(AVPicture * ret, unsigned char ** rows, int lqt_colormodel,
                            int row_span, int row_span_uv)
   {
@@ -175,12 +206,12 @@ static void fill_avpicture(AVPicture * ret, unsigned char ** rows, int lqt_color
     case BC_RGB565:  ///< always stored in cpu endianness
       ret->data[0]     = rows[0];
       ret->linesize[0] = (int)(rows[1] - rows[0]);
-      //      fprintf(stderr, "Linesize: %d\n", ret->linesize[0]);
       break;
     default:
       break;
     }
   }
+#endif
 
 static enum PixelFormat lqt_ffmpeg_get_ffmpeg_colormodel(int id)
   {
@@ -231,7 +262,6 @@ static void convert_image_decode_rgba(AVFrame * in_frame,
   uint32_t * src_ptr;
   uint8_t * dst_ptr;
   int i, j;
-  //  fprintf(stderr, "DECODE RGBA %d %d\n", width, height);
   for(i = 0; i < height; i++)
     {
     src_ptr = (uint32_t*)(in_frame->data[0] + i * in_frame->linesize[0]);
@@ -258,22 +288,48 @@ static void convert_image_decode_rgba(AVFrame * in_frame,
  *  to a libquicktime special one
  */
 
-static void convert_image_decode(AVFrame * in_frame, enum PixelFormat in_format,
+static void convert_image_decode(quicktime_ffmpeg_video_codec_t *codec,
+                                 AVFrame * in_frame, enum PixelFormat in_format,
                                  unsigned char ** out_frame, int out_format,
                                  int width, int height, int row_span, int row_span_uv)
   {
+#ifdef HAVE_LIBSWSCALE
+  uint8_t * out_planes[4];
+  int out_strides[4];
+#else
   AVPicture in_pic;
   AVPicture out_pic;
-
+#endif
+  
   /*
    *  Could someone please tell me, how people can make such a brain dead
    *  RGBA format like in ffmpeg??
    */
+#if LIBAVUTIL_VERSION_INT < (50<<16)
   if((in_format == PIX_FMT_RGBA32) && (out_format == BC_RGBA8888))
+#else
+  if((in_format == PIX_FMT_RGB32) && (out_format == BC_RGBA8888))
+#endif
     {
     convert_image_decode_rgba(in_frame, out_frame, width, height);
     return;
     }
+
+#ifdef HAVE_LIBSWSCALE
+  out_planes[0] = out_frame[0];
+  out_planes[1] = out_frame[0];
+  out_planes[2] = out_frame[0];
+  out_planes[3] = (uint8_t*)0;
+
+  out_strides[0] = row_span;
+  out_strides[1] = row_span_uv;
+  out_strides[2] = row_span_uv;
+  out_strides[3] = 0;
+  
+  sws_scale(codec->swsContext,
+            in_frame->data, in_frame->linesize,
+            0, height, out_planes, out_strides);
+#else
   
   memset(&in_pic,  0, sizeof(in_pic));
   memset(&out_pic, 0, sizeof(out_pic));
@@ -286,14 +342,10 @@ static void convert_image_decode(AVFrame * in_frame, enum PixelFormat in_format,
   in_pic.linesize[2]  = in_frame->linesize[2];
   
   fill_avpicture(&out_pic, out_frame, out_format, row_span, row_span_uv);
-#if 0
-  fprintf(stderr, "img_convert %s -> %s\n", 
-          avcodec_get_pix_fmt_name(in_format),
-          avcodec_get_pix_fmt_name(lqt_ffmpeg_get_ffmpeg_colormodel(out_format)));
-#endif
   img_convert(&out_pic, lqt_ffmpeg_get_ffmpeg_colormodel(out_format),
               &in_pic,  in_format,
               width, height);
+#endif
   
   }
 
@@ -305,7 +357,7 @@ static void convert_image_decode(AVFrame * in_frame, enum PixelFormat in_format,
    doesn't tells us the true colormodel before decoding the first frame */
 
 static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointers,
-                            int track)
+                                   int track)
   {
   uint8_t * user_atom;
   uint32_t user_atom_len;
@@ -327,8 +379,6 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
   int extradata_size = 0;
   
   uint8_t * cpy_rows[3];
-
-  //  fprintf(stderr, "decode video\n");
   
   height = quicktime_video_height(file, track);
   width =  quicktime_video_width(file, track);
@@ -339,7 +389,12 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
     {
     codec->avctx->width           = width;
     codec->avctx->height          = height;
+
+#if LIBAVCODEC_VERSION_INT < ((52<<16)+(0<<8)+0)
     codec->avctx->bits_per_sample = quicktime_video_depth(file, track);
+#else
+    codec->avctx->bits_per_coded_sample = quicktime_video_depth(file, track);
+#endif
 
     /* Set extradata: It's done differently for each codec */
 
@@ -348,43 +403,36 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
       extradata       = trak->mdia.minf.stbl.stsd.table[0].table_raw + 4;
       extradata_size  = trak->mdia.minf.stbl.stsd.table[0].table_raw_size - 4;
       
-      //      fprintf(stderr, "Extradata: %d bytes\n", codec->avctx->extradata_size);
-      //      lqt_hexdump(codec->avctx->extradata, codec->avctx->extradata_size, 16);
       }
     else if(codec->decoder->id == CODEC_ID_H264)
       {
       user_atom = quicktime_stsd_get_user_atom(trak, "avcC", &user_atom_len);
 
       if(!user_atom)
-        fprintf(stderr, "No avcC atom present, decoding is likely to fail\n");
+        lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN,
+                "No avcC atom present, decoding is likely to fail");
       else
         {
         extradata = user_atom + 8;
         extradata_size = user_atom_len - 8;
         }
-      //      fprintf(stderr, "Got avcc atom\n");
-      //      lqt_hexdump(extradata,
-      //                  extradata_size, 16);
       
       }
     else if(codec->decoder->id == CODEC_ID_MPEG4)
       {
       if(trak->mdia.minf.stbl.stsd.table[0].has_esds)
         {
-        //        fprintf(stderr, "Setting MPEG-4 extradata %d bytes\n", 
-        //                trak->mdia.minf.stbl.stsd.table[0].esds.decoderConfigLen);
         
         extradata = trak->mdia.minf.stbl.stsd.table[0].esds.decoderConfig;
         extradata_size = trak->mdia.minf.stbl.stsd.table[0].esds.decoderConfigLen;
         }
-#if 0 /* Hope, that the global header comes with the I-frames */
-      else
-        {
-        fprintf(stderr, "No MPEG-4 extradata!!\n");
-        }
-#endif
       }
-
+    else if((user_atom = quicktime_stsd_get_user_atom(trak, "glbl", &user_atom_len)))
+      {
+      extradata = user_atom + 8;
+      extradata_size = user_atom_len - 8;
+      }
+    
     if(extradata)
       {
       codec->extradata = calloc(1, extradata_size + FF_INPUT_BUFFER_PADDING_SIZE);
@@ -422,41 +470,6 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
     vtrack->stream_cmodel = LQT_COLORMODEL_NONE;
     codec->initialized = 1;
     }
-
-#if 0
-  /* Check if we must seek */
-  if((quicktime_has_keyframes(file, track)) &&
-     (vtrack->current_position != codec->last_frame + 1))
-    {
-    int64_t frame1, frame2 = vtrack->current_position;
-
-    /* Forget about previously decoded frame */
-    codec->have_frame = 0;
-    
-    frame1 = quicktime_get_keyframe_before(file, vtrack->current_position, track);
-    if((frame1 < codec->last_frame) &&
-       (frame2 > codec->last_frame))
-      frame1 = codec->last_frame + 1;
-    while(frame1 < frame2)
-      {
-      buffer_size = lqt_read_video_frame(file, &codec->buffer,
-                                         &codec->buffer_alloc,
-                                         frame1, track);
-      if(buffer_size > 0)
-        {
-        avcodec_decode_video(codec->avctx,
-                             codec->frame,
-                             &got_pic,
-                             codec->buffer,
-                             buffer_size);
-        }
-      frame1++;
-      }
-    vtrack->current_position = frame2;
-    }
-
-  codec->last_frame = vtrack->current_position;
-#endif
   
   /* Read the frame from file and decode it */
 
@@ -470,29 +483,37 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
                                          &codec->buffer_alloc,
                                          vtrack->current_position + codec->decoding_delay,
                                          NULL, track);
-      
-      if(buffer_size <= 0)
-        return 0;
-#if 0
-      fprintf(stderr, "Decode video...");
-      fprintf(stderr, "Frame size: %d pos: %lld\n", buffer_size, vtrack->current_position);
-      lqt_hexdump(codec->buffer, 16, 16);
-      
-#endif 
+
+      codec->decoding_delay++;
+
+#if LIBAVCODEC_BUILD >= ((52<<16)+(26<<8)+0)
+      codec->pkt.data = codec->buffer;
+      codec->pkt.size = buffer_size;
+      if(avcodec_decode_video2(codec->avctx,
+                              codec->frame,
+                              &got_pic,
+                              &codec->pkt) < 0)
+        {
+        lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "Skipping corrupted frame");
+        continue;
+        }
+#else
+    
       if(avcodec_decode_video(codec->avctx,
                               codec->frame,
                               &got_pic,
                               codec->buffer,
                               buffer_size) < 0)
         {
-        fprintf(stderr, "Skipping corrupted frame\n");
+        lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "Skipping corrupted frame");
         continue;
         }
-      //      fprintf(stderr, "done\n");
-
-      if(!got_pic && trak->mdia.minf.stbl.has_ctts)
-        codec->decoding_delay++;
+#endif      
+      if(got_pic)
+        codec->decoding_delay--;
       
+      if((buffer_size <= 0) && !got_pic)
+        return 0;
       }
     }
   
@@ -501,18 +522,42 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
     vtrack->stream_cmodel =
       lqt_ffmpeg_get_lqt_colormodel(codec->avctx->pix_fmt, &exact);
     if(!exact)
+      {
       codec->do_imgconvert = 1;
-    //    fprintf(stderr, "Detected stream colormodel: %s %s %d\n",
-    //            lqt_colormodel_to_string(codec->lqt_colormodel),
-    //            avcodec_get_pix_fmt_name(codec->avctx->pix_fmt), exact);
+      
+#ifdef HAVE_LIBSWSCALE
 
+#if LIBAVUTIL_VERSION_INT < (50<<16)
+      if(!((codec->avctx->pix_fmt == PIX_FMT_RGBA32) &&
+           (vtrack->stream_cmodel == BC_RGBA8888)))
+#else
+      if(!((codec->avctx->pix_fmt == PIX_FMT_RGB32) &&
+           (vtrack->stream_cmodel == BC_RGBA8888)))
+#endif
+        {
+        codec->swsContext =
+          sws_getContext(width, height,
+                         codec->avctx->pix_fmt,
+                         width, height,
+                         lqt_ffmpeg_get_ffmpeg_colormodel(vtrack->stream_cmodel),
+                         0, (SwsFilter*)0,
+                         (SwsFilter*)0,
+                         (double*)0);
+        }
+#endif
+      }
     if(codec->decoder->id == CODEC_ID_DVVIDEO)
       {
       if(vtrack->stream_cmodel == BC_YUV420P)
         vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_DVPAL;
       vtrack->interlace_mode = LQT_INTERLACE_BOTTOM_FIRST;
       }
-
+    else if((codec->decoder->id == CODEC_ID_MPEG4) ||
+            (codec->decoder->id == CODEC_ID_H264))
+      {
+      if(vtrack->stream_cmodel == BC_YUV420P)
+        vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_MPEG2;
+      }
     if(codec->avctx->sample_aspect_ratio.num)
       {
       trak->mdia.minf.stbl.stsd.table[0].pasp.hSpacing = codec->avctx->sample_aspect_ratio.num;
@@ -552,13 +597,11 @@ static int lqt_ffmpeg_decode_video(quicktime_t *file, unsigned char **row_pointe
     }
   else
     {
-    //    fprintf(stderr, "img_convert %d...", codec->avctx->pix_fmt);
-    convert_image_decode(codec->frame, codec->avctx->pix_fmt,
+    convert_image_decode(codec, codec->frame, codec->avctx->pix_fmt,
                          row_pointers, vtrack->stream_cmodel,
                          width, height, vtrack->stream_row_span,
                          vtrack->stream_row_span_uv);
     }
-  //  fprintf(stderr, "Done\n"); 
   codec->have_frame = 0;
   return result;
   }
@@ -578,7 +621,6 @@ static void resync_ffmpeg(quicktime_t *file, int track)
   /* Reset lavc */
   avcodec_flush_buffers(codec->avctx);
 
-  //  fprintf(stderr, "Resync: %ld\n", vtrack->current_position);
   
   if(quicktime_has_keyframes(file, track))
     {
@@ -593,11 +635,20 @@ static void resync_ffmpeg(quicktime_t *file, int track)
                                          frame, NULL, track);
       if(buffer_size > 0)
         {
+#if LIBAVCODEC_BUILD >= ((52<<16)+(26<<8)+0)
+        codec->pkt.data = codec->buffer;
+        codec->pkt.size = buffer_size;
+        avcodec_decode_video2(codec->avctx,
+                              codec->frame,
+                              &got_pic,
+                              &codec->pkt);
+#else
         avcodec_decode_video(codec->avctx,
                              codec->frame,
                              &got_pic,
                              codec->buffer,
                              buffer_size);
+#endif
         }
       frame++;
       }
@@ -617,7 +668,6 @@ static int set_pass_ffmpeg(quicktime_t *file,
   codec->stats_filename = malloc(strlen(stats_file)+1);
   strcpy(codec->stats_filename, stats_file);
   
-  fprintf(stderr, "set_pass_ffmpeg %d %d %s\n", pass, total_passes, stats_file);
   return 1;
   }
 
@@ -633,11 +683,29 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
 	quicktime_ffmpeg_video_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
 	int height = trak->tkhd.track_height;
 	int width = trak->tkhd.track_width;
-        quicktime_atom_t chunk_atom;
         int stats_len;
         if(!row_pointers)
           {
-          vtrack->stream_cmodel = codec->encode_colormodel;
+          if(vtrack->stream_cmodel == BC_YUV420P)
+            {
+            if(codec->encoder->id == CODEC_ID_MPEG4)
+              {
+              vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_MPEG2;
+#ifndef DO_INTERLACE // Seems not to work yet
+            /* enable interlaced encoding */
+              vtrack->interlace_mode = LQT_INTERLACE_NONE;
+#endif
+              }
+            else if(codec->encoder->id == CODEC_ID_DVVIDEO)
+              {
+              vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_DVPAL;
+              }
+#ifndef DO_INTERLACE // Seems not to work yet
+            else
+              /* enable interlaced encoding */
+              vtrack->interlace_mode = LQT_INTERLACE_NONE;
+#endif
+            }
           return 0;
           }
         
@@ -655,10 +723,7 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
 
           if(codec->avctx->flags & CODEC_FLAG_QSCALE)
             codec->avctx->global_quality = codec->qscale;
-          
-          //          fprintf(stderr, "Init %d %d\n", width, height);
-          
-          
+                              
           codec->avctx->width = width;
           codec->avctx->height = height;
                 
@@ -682,11 +747,44 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
               strncpy(trak->strl->strh.fccHandler, "divx", 4);
               strncpy(trak->strl->strf.bh.biCompression, "DX50", 4);
               }
+#ifdef DO_INTERLACE // Seems not to work yet
+            /* enable interlaced encoding */
+            if(vtrack->interlace_mode != LQT_INTERLACE_NONE)
+              {
+              lqt_log(file, LQT_LOG_INFO, LOG_DOMAIN, "Enabling interlaced encoding");
+              codec->avctx->flags |=
+                (CODEC_FLAG_INTERLACED_DCT|CODEC_FLAG_INTERLACED_ME|CODEC_FLAG_ALT_SCAN);
+              }
+#endif
             }
-          else if((codec->encoder->id == CODEC_ID_MSMPEG4V3) && (trak->strl))
+          else if((codec->encoder->id == CODEC_ID_MSMPEG4V3) && (trak->strl) &&
+                  !strncmp(trak->strl->strf.bh.biCompression, "DIV3", 4))
             {
             strncpy(trak->strl->strh.fccHandler, "div3", 4);
             }
+          else if((codec->encoder->id == CODEC_ID_H263) &&
+                  (file->file_type & (LQT_FILE_MP4|LQT_FILE_3GP)))
+            {
+            uint8_t d263_data[] =
+              { 'l', 'q', 't', ' ', /* Vendor? */
+                0, /* Decoder version */
+                0xa, /* Level */
+                0 /* Profile */ };
+            quicktime_user_atoms_add_atom(&(trak->mdia.minf.stbl.stsd.table[0].user_atoms),
+                                          "d263", d263_data,
+                                          sizeof(d263_data));
+            strncpy(trak->mdia.minf.stbl.stsd.table[0].format,
+                    "s263", 4);
+            }
+          else if(codec->encoder->id == CODEC_ID_FFVHUFF)
+            {
+            if(!(file->file_type & (LQT_FILE_AVI|LQT_FILE_AVI_ODML)))
+              {
+              codec->avctx->flags |= CODEC_FLAG_GLOBAL_HEADER;
+              codec->write_global_header = 1;
+              }
+            }
+          
           /* Initialize 2-pass */
           if(codec->total_passes)
             {
@@ -719,18 +817,12 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
           codec->buffer = malloc(codec->buffer_alloc);
           if(!codec->buffer)
             return -1;
-
-          if(codec->avctx->max_b_frames > 0)
-            vtrack->has_b_frames = 1;
           
           codec->initialized = 1;
-
-          
           
 	}
         //        codec->lqt_colormodel = ffmepg_2_lqt(codec->com.ffcodec_enc);
         
-        //        fprintf(stderr, "Encode, PTS: %lld...\n", vtrack->timestamp);
         codec->frame->data[0] = row_pointers[0];
         codec->frame->data[1] = row_pointers[1];
         codec->frame->data[2] = row_pointers[2];
@@ -740,84 +832,79 @@ static int lqt_ffmpeg_encode_video(quicktime_t *file, unsigned char **row_pointe
         codec->frame->pts = vtrack->timestamp;
         if(codec->avctx->flags & CODEC_FLAG_QSCALE)
           codec->frame->quality = codec->qscale;
-        //        fprintf(stderr, "Encode %p, %d\n", codec->buffer,
-        //                codec->buffer_size);
-        
+#ifdef DO_INTERLACE
+        if(vtrack->interlace_mode != LQT_INTERLACE_NONE)
+          {
+          codec->frame->interlaced_frame = 1;
+          if(vtrack->interlace_mode == LQT_INTERLACE_TOP_FIRST)
+            codec->frame->top_field_first = 1;
+          }
+#endif   
 	bytes_encoded = avcodec_encode_video(codec->avctx,
                                              codec->buffer,
                                              codec->buffer_alloc,
                                              codec->frame);
-#if 0
-        fprintf(stderr, "Encoded %d bytes, ", bytes_encoded);
-        switch(codec->avctx->coded_frame->pict_type)
-          {
-          case FF_I_TYPE:
-            fprintf(stderr, "pict_type: I, ");
-            break;
-          case FF_P_TYPE:
-            fprintf(stderr, "pict_type: P, ");
-            break;
-          case FF_B_TYPE:
-            fprintf(stderr, "pict_type: B, ");
-            break;
-          }
-        fprintf(stderr, "PTS: %lld\n", codec->avctx->coded_frame->pts);
-#endif
 
+        if(bytes_encoded < 0)
+          return -1;
+        
         if(bytes_encoded)
           {
-          vtrack->coded_timestamp = codec->avctx->coded_frame->pts;
-          quicktime_write_chunk_header(file, trak, &chunk_atom);
+          lqt_write_frame_header(file, track,
+                                 -1, codec->avctx->coded_frame->pts,
+                                 codec->avctx->coded_frame->key_frame);
           
           result = !quicktime_write_data(file, 
                                          codec->buffer, 
                                          bytes_encoded);
-          quicktime_write_chunk_footer(file, 
-                                       trak, 
-                                       vtrack->current_chunk,
-                                       &chunk_atom, 
-                                       1);
-          
-          if(codec->avctx->coded_frame->key_frame)
-            quicktime_insert_keyframe(file, vtrack->current_chunk-1, track);
-          vtrack->current_chunk++;
 
+          lqt_write_frame_footer(file, track);
+          
           /* Write stats */
           
           if((codec->pass == 1) && codec->avctx->stats_out && codec->stats_file)
-            fprintf(codec->stats_file, codec->avctx->stats_out);
+            fprintf(codec->stats_file, "%s", codec->avctx->stats_out);
           }
         
         /* Check whether to write the global header */
-        
+
         if(codec->write_global_header && !codec->global_header_written)
           {
-          esds = quicktime_set_esds(trak,
-                                    codec->avctx->extradata,
-                                    codec->avctx->extradata_size);
-          //          fprintf(stderr, "Setting MPEG-4 extradata\n");
-          //          lqt_hexdump(codec->avctx->extradata, codec->avctx->extradata_size,
-          //                      16);
+          if(codec->encoder->id == CODEC_ID_FFVHUFF)
+            {
+            quicktime_user_atoms_add_atom(&(trak->mdia.minf.stbl.stsd.table[0].user_atoms),
+                                          "glbl",
+                                          codec->avctx->extradata, codec->avctx->extradata_size );
+            }
+          else if(codec->encoder->id == CODEC_ID_MPEG4)
+            {
+            esds = quicktime_set_esds(trak,
+                                      codec->avctx->extradata,
+                                      codec->avctx->extradata_size);
           
-          esds->version         = 0;
-          esds->flags           = 0;
+            esds->version         = 0;
+            esds->flags           = 0;
           
-          esds->esid            = 0;
-          esds->stream_priority = 0;
+            esds->esid            = 0;
+            esds->stream_priority = 0;
           
-          esds->objectTypeId    = 32; /* MPEG-4 video */
-          esds->streamType      = 0x11; /* from qt4l and CDR_Dinner_350k-994.mp4 */
-          esds->bufferSizeDB    = 64000; /* Hopefully not important :) */
+            esds->objectTypeId    = 32; /* MPEG-4 video */
+            esds->streamType      = 0x11; /* from qt4l and CDR_Dinner_350k-994.mp4 */
+            esds->bufferSizeDB    = 64000; /* Hopefully not important :) */
           
-          /* Maybe correct these later? */
-          esds->maxBitrate      = 200000;
-          esds->avgBitrate      = 200000;
+            /* Maybe correct these later? */
+            esds->maxBitrate      = 200000;
+            esds->avgBitrate      = 200000;
+          
+            /* Set iods profile */
+            if(codec->avctx->max_b_frames ||
+               (codec->avctx->flags & (CODEC_FLAG_QPEL|CODEC_FLAG_GMC)))
+              file->moov.iods.videoProfileId = 0xf3; // Advanced Simple Profile @ Level 3
+            else
+              file->moov.iods.videoProfileId = 0x03; // Simple Profile @ Level 3
+            }
           codec->global_header_written = 1;
-          
-          /* Set iods profile */
-          file->moov.iods.videoProfileId = 0xf3; // MPEG-4 Adv Simple@L3
           }
-
         return result;
 }
 
@@ -826,9 +913,7 @@ static int flush(quicktime_t *file, int track)
         int result = 0;
 	int bytes_encoded;
 	quicktime_video_map_t *vtrack = &(file->vtracks[track]);
-	quicktime_trak_t *trak = vtrack->track;
 	quicktime_ffmpeg_video_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
-        quicktime_atom_t chunk_atom;
 
         /* Do nothing if we didn't encode anything yet */
         if(!codec->initialized)
@@ -838,43 +923,24 @@ static int flush(quicktime_t *file, int track)
                                              codec->buffer,
                                              codec->buffer_alloc,
                                              (AVFrame*)0);
-#if 0
-        fprintf(stderr, "Flush: encoded %d bytes, ", bytes_encoded);
-        switch(codec->avctx->coded_frame->pict_type)
-          {
-          case FF_I_TYPE:
-            fprintf(stderr, "pict_type: I, ");
-            break;
-          case FF_P_TYPE:
-            fprintf(stderr, "pict_type: P, ");
-            break;
-          case FF_B_TYPE:
-            fprintf(stderr, "pict_type: B, ");
-            break;
-          }
-        fprintf(stderr, "PTS: %lld\n", codec->avctx->coded_frame->pts);
-#endif
-        vtrack->coded_timestamp = codec->avctx->coded_frame->pts;
 
+        if(bytes_encoded < 0)
+          return 0;
+        
         if(bytes_encoded)
           {
-          quicktime_write_chunk_header(file, trak, &chunk_atom);
+          lqt_write_frame_header(file, track,
+                                 -1, codec->avctx->coded_frame->pts,
+                                 codec->avctx->coded_frame->key_frame);
           
           result = !quicktime_write_data(file, 
                                          codec->buffer, 
                                          bytes_encoded);
-          quicktime_write_chunk_footer(file, 
-                                       trak, 
-                                       vtrack->current_chunk,
-                                       &chunk_atom, 
-                                       1);
-          
-          if(codec->avctx->coded_frame->key_frame)
-            quicktime_insert_keyframe(file, vtrack->current_chunk-1, track);
-          vtrack->current_chunk++;
 
+          lqt_write_frame_footer(file, track);
+          
           if((codec->pass == 1) && codec->avctx->stats_out && codec->stats_file)
-            fprintf(codec->stats_file, codec->avctx->stats_out);
+            fprintf(codec->stats_file, "%s", codec->avctx->stats_out);
           
           return 1;
           }
@@ -884,8 +950,8 @@ static int flush(quicktime_t *file, int track)
 
 static int set_parameter_video(quicktime_t *file, 
                                int track, 
-                               char *key, 
-                               void *value)
+                               const char *key, 
+                               const void *value)
   {
   quicktime_ffmpeg_video_codec_t *codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;
 #if 1
@@ -915,16 +981,19 @@ void quicktime_init_video_codec_ffmpeg(quicktime_video_map_t *vtrack, AVCodec *e
         codec->avctx = avcodec_alloc_context();
         
 	if (quicktime_match_32(compressor, "dvc "))
-	   codec->encode_colormodel = BC_YUV411P;
+	   vtrack->stream_cmodel = BC_YUV411P;
         else if (quicktime_match_32(compressor, "dv5n") ||
-                 quicktime_match_32(compressor, "dv5p"))
-            codec->encode_colormodel = BC_YUV422P;
+                 quicktime_match_32(compressor, "dv5p") ||
+                 quicktime_match_32(compressor, "AVdn"))
+          vtrack->stream_cmodel = BC_YUV422P;
         else if (quicktime_match_32(compressor, "MJPG"))
-          codec->encode_colormodel = BC_YUVJ420P;
+          vtrack->stream_cmodel = BC_YUVJ420P;
+        else if (quicktime_match_32(compressor, "rle "))
+          vtrack->stream_cmodel = BC_RGB888;
         //        else if(quicktime_match_32(compressor, "dvcp"))
         //          codec->encode_colormodel = BC_YUV411P;
         else
-          codec->encode_colormodel = BC_YUV420P;
+          vtrack->stream_cmodel = BC_YUV420P;
         
 	codec->encoder = encoder;
 	codec->decoder = decoder;
