@@ -147,15 +147,12 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
             }
           }
         
-	quicktime_write_chunk_header(file, trak, &chunk_atom);
-	result = !quicktime_write_data(file, codec->buffer, bytes);
-	quicktime_write_chunk_footer(file, 
-		trak,
-		vtrack->current_chunk,
-		&chunk_atom, 
-		1);
+        lqt_write_frame_header(file, track,
+                               vtrack->current_position,
+                               -1, 0);
 
-	vtrack->current_chunk++;
+	result = !quicktime_write_data(file, codec->buffer, bytes);
+        lqt_write_frame_footer(file, track);
 	
 	return result;
 }
