@@ -1,6 +1,31 @@
-#include "funcprotos.h"
+/*******************************************************************************
+ yuv4.c
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
 #include <quicktime/colormodels.h>
-#include "quicktime.h"
+#include <stdlib.h>
+#include "videocodec.h"
 
 /* U V values are signed but Y R G B values are unsigned! */
 /*
@@ -343,21 +368,18 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 		}
 	}
 
-	quicktime_write_chunk_header(file, trak, &chunk_atom);
+        lqt_write_frame_header(file, track,
+                               vtrack->current_position,
+                               -1, 0);
+
 	result = quicktime_write_data(file, buffer, bytes);
 	if(result)
 		result = 0; 
 	else 
 		result = 1;
-	quicktime_write_chunk_footer(file, 
-		trak,
-		vtrack->current_chunk,
-		&chunk_atom, 
-		1);
-
-
-	vtrack->current_chunk++;
-	return result;
+        
+        lqt_write_frame_footer(file, track);
+        return result;
 }
 
 
