@@ -1,103 +1,132 @@
-#include <string.h>
+/*******************************************************************************
+ x264.c
 
-#include "funcprotos.h"
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
 
-#include <quicktime/lqt.h>
-#include <quicktime/colormodels.h>
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
 
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
 #include <x264.h>
+#include <stdlib.h>
+#include <string.h>
+#include <quicktime/colormodels.h>
+#include "qtx264.h"
+
+#define LOG_DOMAIN "x264"
 
 // #define DUMP_CONFIG
 #ifdef DUMP_CONFIG
 
+
 static void dump_params(x264_param_t * params)
   {
-  fprintf(stderr, "X264 params:\n");
-  fprintf(stderr, "  cpu:       %08x\n", params->cpu);
-  fprintf(stderr, "  i_threads: %d\n", params->i_threads);
-  fprintf(stderr, "  i_width: %d\n", params->i_width);
-  fprintf(stderr, "  i_height: %d\n", params->i_height);
-  fprintf(stderr, "  i_csp: %d\n", params->i_csp);
-  fprintf(stderr, "  i_level_idc: %d\n", params->i_level_idc);
-  fprintf(stderr, "  i_frame_total: %d\n", params->i_frame_total);
-  fprintf(stderr, "  vui:\n");
-  fprintf(stderr, "    i_sar_height: %d\n", params->vui.i_sar_height);
-  fprintf(stderr, "    i_sar_width:  %d\n", params->vui.i_sar_width);
-  fprintf(stderr, "    i_overscan:   %d\n", params->vui.i_overscan);
-  fprintf(stderr, "    i_vidformat:  %d\n", params->vui.i_vidformat);
-  fprintf(stderr, "    b_fullrange:  %d\n", params->vui.b_fullrange);
-  fprintf(stderr, "    i_colorprim:  %d\n", params->vui.i_colorprim);
-  fprintf(stderr, "    i_transfer:   %d\n", params->vui.i_transfer);
-  fprintf(stderr, "    i_colmatrix:  %d\n", params->vui.i_colmatrix);
-  fprintf(stderr, "    i_chroma_loc: %d\n", params->vui.i_chroma_loc);
+  lqt_dump("X264 params:\n");
+  lqt_dump("  cpu:       %08x\n", params->cpu);
+  lqt_dump("  i_threads: %d\n", params->i_threads);
+  lqt_dump("  i_width: %d\n", params->i_width);
+  lqt_dump("  i_height: %d\n", params->i_height);
+  lqt_dump("  i_csp: %d\n", params->i_csp);
+  lqt_dump("  i_level_idc: %d\n", params->i_level_idc);
+  lqt_dump("  i_frame_total: %d\n", params->i_frame_total);
+  lqt_dump("  vui:\n");
+  lqt_dump("    i_sar_height: %d\n", params->vui.i_sar_height);
+  lqt_dump("    i_sar_width:  %d\n", params->vui.i_sar_width);
+  lqt_dump("    i_overscan:   %d\n", params->vui.i_overscan);
+  lqt_dump("    i_vidformat:  %d\n", params->vui.i_vidformat);
+  lqt_dump("    b_fullrange:  %d\n", params->vui.b_fullrange);
+  lqt_dump("    i_colorprim:  %d\n", params->vui.i_colorprim);
+  lqt_dump("    i_transfer:   %d\n", params->vui.i_transfer);
+  lqt_dump("    i_colmatrix:  %d\n", params->vui.i_colmatrix);
+  lqt_dump("    i_chroma_loc: %d\n", params->vui.i_chroma_loc);
 
-  fprintf(stderr, "  fps: %d:%d\n", params->i_fps_num, params->i_fps_den);
+  lqt_dump("  fps: %d:%d\n", params->i_fps_num, params->i_fps_den);
 
   /* Bitstream parameters */
   
-  fprintf(stderr, "  i_frame_reference:           %d\n", params->i_frame_reference); // 1..16 
-  fprintf(stderr, "  i_keyint_min:                %d\n", params->i_keyint_min);      
-  fprintf(stderr, "  i_keyint_max:                %d\n", params->i_keyint_max);
-  fprintf(stderr, "  i_scenecut_threshold:        %d\n", params->i_scenecut_threshold); 
-  fprintf(stderr, "  i_bframe:                    %d\n", params->i_bframe);          // 0.. X264_BFRAME_MAX
-  fprintf(stderr, "  b_bframe_adaptive:           %d\n", params->b_bframe_adaptive);
-  fprintf(stderr, "  i_bframe_bias:               %d\n", params->i_bframe_bias);
-  fprintf(stderr, "  b_bframe_pyramid:            %d\n", params->b_bframe_pyramid);
+  lqt_dump("  i_frame_reference:           %d\n", params->i_frame_reference); // 1..16 
+  lqt_dump("  i_keyint_min:                %d\n", params->i_keyint_min);      
+  lqt_dump("  i_keyint_max:                %d\n", params->i_keyint_max);
+  lqt_dump("  i_scenecut_threshold:        %d\n", params->i_scenecut_threshold); 
+  lqt_dump("  i_bframe:                    %d\n", params->i_bframe);          // 0.. X264_BFRAME_MAX
+  lqt_dump("  b_bframe_adaptive:           %d\n", params->b_bframe_adaptive);
+  lqt_dump("  i_bframe_bias:               %d\n", params->i_bframe_bias);
+  lqt_dump("  b_bframe_pyramid:            %d\n", params->b_bframe_pyramid);
   
-  fprintf(stderr, "  b_deblocking_filter:         %d\n", params->b_deblocking_filter);
-  fprintf(stderr, "  i_deblocking_filter_alphac0: %d\n", params->i_deblocking_filter_alphac0); // -6..6
-  fprintf(stderr, "  i_deblocking_filter_beta:    %d\n", params->i_deblocking_filter_beta);    // -6..6
+  lqt_dump("  b_deblocking_filter:         %d\n", params->b_deblocking_filter);
+  lqt_dump("  i_deblocking_filter_alphac0: %d\n", params->i_deblocking_filter_alphac0); // -6..6
+  lqt_dump("  i_deblocking_filter_beta:    %d\n", params->i_deblocking_filter_beta);    // -6..6
 
-  fprintf(stderr, "  b_cabac:                     %d\n", params->b_cabac);
-  fprintf(stderr, "  i_cabac_init_idc:            %d\n", params->i_cabac_init_idc); // 0..2
+  lqt_dump("  b_cabac:                     %d\n", params->b_cabac);
+  lqt_dump("  i_cabac_init_idc:            %d\n", params->i_cabac_init_idc); // 0..2
 
-  fprintf(stderr, "  i_cqm_preset:                %d\n", params->i_cqm_preset);  
-  fprintf(stderr, "  psz_cqm_file:                %s\n", params->psz_cqm_file);
+  lqt_dump("  i_cqm_preset:                %d\n", params->i_cqm_preset);  
+  lqt_dump("  psz_cqm_file:                %s\n", params->psz_cqm_file);
 
-  fprintf(stderr, "  Analyze:\n");
+  lqt_dump("  Analyze:\n");
 
   // ORED value of X264_ANALYSE_I4x4, X264_ANALYSE_I8x8, X264_ANALYSE_PSUB16x16, X264_ANALYSE_PSUB8x8,
   // X264_ANALYSE_BSUB16x16
   
-  fprintf(stderr, "    intra:              %d\n", params->analyse.intra);
-  fprintf(stderr, "    inter:              %d\n", params->analyse.inter);
+  lqt_dump("    intra:              %d\n", params->analyse.intra);
+  lqt_dump("    inter:              %d\n", params->analyse.inter);
   
   
-  fprintf(stderr, "    b_transform_8x8:    %d\n", params->analyse.b_transform_8x8);
-  fprintf(stderr, "    b_weighted_bipred:  %d\n", params->analyse.b_weighted_bipred);
-  fprintf(stderr, "    i_direct_mv_pred:   %d\n", params->analyse.i_direct_mv_pred);
-  fprintf(stderr, "    i_chroma_qp_offset: %d\n", params->analyse.i_chroma_qp_offset);
+  lqt_dump("    b_transform_8x8:    %d\n", params->analyse.b_transform_8x8);
+  lqt_dump("    b_weighted_bipred:  %d\n", params->analyse.b_weighted_bipred);
+  lqt_dump("    i_direct_mv_pred:   %d\n", params->analyse.i_direct_mv_pred);
+  lqt_dump("    i_chroma_qp_offset: %d\n", params->analyse.i_chroma_qp_offset);
 
   // X264_ME_DIA, X264_ME_HEX, X264_ME_UMH, X264_ME_ESA
-  fprintf(stderr, "    i_me_method:        %d\n", params->analyse.i_me_method); 
+  lqt_dump("    i_me_method:        %d\n", params->analyse.i_me_method); 
 
   
-  fprintf(stderr, "    i_me_range:         %d\n", params->analyse.i_me_range);
-  fprintf(stderr, "    i_mv_range:         %d\n", params->analyse.i_mv_range);
-  fprintf(stderr, "    i_subpel_refine:    %d\n", params->analyse.i_subpel_refine); // 1..7
-  fprintf(stderr, "    b_bidir_me:         %d\n", params->analyse.b_bidir_me);
-  fprintf(stderr, "    b_chroma_me:        %d\n", params->analyse.b_chroma_me);
-  fprintf(stderr, "    b_bframe_rdo:       %d\n", params->analyse.b_bframe_rdo);
-  fprintf(stderr, "    b_mixed_references: %d\n", params->analyse.b_mixed_references);
-  fprintf(stderr, "    i_trellis:          %d\n", params->analyse.i_trellis);         // 0..2
-  fprintf(stderr, "    b_fast_pskip:       %d\n", params->analyse.b_fast_pskip);
-  fprintf(stderr, "    i_noise_reduction:  %d\n", params->analyse.i_noise_reduction); // 0..1<<16
-  fprintf(stderr, "    b_psnr:             %d\n", params->analyse.b_psnr);
+  lqt_dump("    i_me_range:         %d\n", params->analyse.i_me_range);
+  lqt_dump("    i_mv_range:         %d\n", params->analyse.i_mv_range);
+  lqt_dump("    i_subpel_refine:    %d\n", params->analyse.i_subpel_refine); // 1..7
+  lqt_dump("    b_bidir_me:         %d\n", params->analyse.b_bidir_me);
+  lqt_dump("    b_chroma_me:        %d\n", params->analyse.b_chroma_me);
+  lqt_dump("    b_bframe_rdo:       %d\n", params->analyse.b_bframe_rdo);
+  lqt_dump("    b_mixed_references: %d\n", params->analyse.b_mixed_references);
+  lqt_dump("    i_trellis:          %d\n", params->analyse.i_trellis);         // 0..2
+  lqt_dump("    b_fast_pskip:       %d\n", params->analyse.b_fast_pskip);
+  lqt_dump("    i_noise_reduction:  %d\n", params->analyse.i_noise_reduction); // 0..1<<16
+  lqt_dump("    b_psnr:             %d\n", params->analyse.b_psnr);
 
-  fprintf(stderr, "  Rate control:\n");
-  fprintf(stderr, "    i_qp_constant:      %d\n", params->rc.i_qp_constant);
-  fprintf(stderr, "    i_qp_min:           %d\n", params->rc.i_qp_min);
-  fprintf(stderr, "    i_qp_max:           %d\n", params->rc.i_qp_max);
-  fprintf(stderr, "    i_qp_step:          %d\n", params->rc.i_qp_step);
-  fprintf(stderr, "    b_cbr:              %d\n", params->rc.b_cbr);
-  fprintf(stderr, "    i_bitrate:          %d\n", params->rc.i_bitrate);
-  fprintf(stderr, "    i_rf_constant:      %d\n", params->rc.i_rf_constant);
-  fprintf(stderr, "    f_rate_tolerance:   %f\n", params->rc.f_rate_tolerance);
-  fprintf(stderr, "    i_vbv_max_bitrate:  %d\n", params->rc.i_vbv_max_bitrate);
-  fprintf(stderr, "    i_vbv_buffer_size:  %d\n", params->rc.i_vbv_buffer_size);
-  fprintf(stderr, "    f_vbv_buffer_init:  %f\n", params->rc.f_vbv_buffer_init);
-  fprintf(stderr, "    f_ip_factor:        %f\n", params->rc.f_ip_factor);
-  fprintf(stderr, "    f_pb_factor:        %f\n", params->rc.f_pb_factor);
+  lqt_dump("  Rate control:\n");
+  lqt_dump("    i_rc_method:        %d\n", params->rc.i_rc_method);
+  lqt_dump("    i_qp_constant:      %d\n", params->rc.i_qp_constant);
+  lqt_dump("    i_qp_min:           %d\n", params->rc.i_qp_min);
+  lqt_dump("    i_qp_max:           %d\n", params->rc.i_qp_max);
+  lqt_dump("    i_qp_step:          %d\n", params->rc.i_qp_step);
+  lqt_dump("    i_bitrate:          %d\n", params->rc.i_bitrate);
+#if X264_BUILD < 54
+  lqt_dump("    i_rf_constant:      %d\n", params->rc.i_rf_constant);
+#else
+  lqt_dump("    f_rf_constant:      %f\n", params->rc.f_rf_constant);
+#endif
+  lqt_dump("    f_rate_tolerance:   %f\n", params->rc.f_rate_tolerance);
+  lqt_dump("    i_vbv_max_bitrate:  %d\n", params->rc.i_vbv_max_bitrate);
+  lqt_dump("    i_vbv_buffer_size:  %d\n", params->rc.i_vbv_buffer_size);
+  lqt_dump("    f_vbv_buffer_init:  %f\n", params->rc.f_vbv_buffer_init);
+  lqt_dump("    f_ip_factor:        %f\n", params->rc.f_ip_factor);
+  lqt_dump("    f_pb_factor:        %f\n", params->rc.f_pb_factor);
   
   }
 
@@ -160,7 +189,7 @@ encode_nals(uint8_t *buf, int size, x264_nal_t *nals, int nnal)
 
 static uint8_t *avc_find_startcode( uint8_t *p, uint8_t *end )
   {
-  uint8_t *a = p + 4 - ((int)p & 3);
+  uint8_t *a = p + 4 - ((long)p & 3);
   
   for( end -= 3; p < a && p < end; p++ )
     {
@@ -275,7 +304,6 @@ static uint8_t * create_avcc_atom(quicktime_x264_codec_t * codec,
   uint8_t nal_type;
   
   x264_encoder_headers(codec->enc, &nal, &nnal);
-  //  fprintf(stderr, "Encoded %d header nals\n", nnal);
 
   tmp_size = 0;
 
@@ -325,8 +353,6 @@ static uint8_t * create_avcc_atom(quicktime_x264_codec_t * codec,
     2 +        // pps_size
     pps_size;  // pps
 
-  //  fprintf(stderr, "ret_size: %d, sps_size: %d, pps_size: %d\n",
-  //          *ret_size, sps_size, pps_size);
   
   ret = malloc(*ret_size);
   ret_ptr = ret;
@@ -371,7 +397,6 @@ static int flush_frame(quicktime_t *file, int track,
                        x264_picture_t * pic_in)
   {
   int result;
-  quicktime_atom_t chunk_atom;
   x264_nal_t *nal;
   int nnal;
   x264_picture_t pic_out;
@@ -379,7 +404,7 @@ static int flush_frame(quicktime_t *file, int track,
 
   quicktime_video_map_t *vtrack = &(file->vtracks[track]);
   quicktime_x264_codec_t *codec = ((quicktime_codec_t*)vtrack->codec)->priv;
-  quicktime_trak_t *trak = vtrack->track;
+
   pic_out.i_pts = 0;
   /* Encode frames, get nals */
   if(x264_encoder_encode(codec->enc, &nal, &nnal, pic_in, &pic_out))
@@ -399,26 +424,18 @@ static int flush_frame(quicktime_t *file, int track,
   if(encoded_size < 0)
     return 0;
 
-  vtrack->coded_timestamp =   pic_out.i_pts;
-
-  //  fprintf(stderr, "x264: Encoded %d bytes, pts: %lld\n", encoded_size, pic_out.i_pts);
-
   if(encoded_size)
     {
-    quicktime_write_chunk_header(file, trak, &chunk_atom);
-  
+    lqt_write_frame_header(file, track,
+                           -1, pic_out.i_pts,
+                           pic_out.i_type == X264_TYPE_IDR);
+    
     result = !quicktime_write_data(file, 
                                    codec->work_buffer_1, 
                                    encoded_size);
-    quicktime_write_chunk_footer(file, 
-                                 trak, 
-                                 vtrack->current_chunk,
-                                 &chunk_atom, 
-                                 1);
-  
-    if(pic_out.i_type == X264_TYPE_IDR)
-      quicktime_insert_keyframe(file, vtrack->current_chunk-1, track);
-    vtrack->current_chunk++;
+
+    lqt_write_frame_footer(file, track);
+    
     return 1;
     }
   return 0;
@@ -437,10 +454,92 @@ static int set_pass_x264(quicktime_t *file,
   codec->stats_filename = malloc(strlen(stats_file)+1);
   strcpy(codec->stats_filename, stats_file);
   
-  //  fprintf(stderr, "set_pass_x264 %d %d %s\n", pass, total_passes, stats_file);
   return 1;
   }
 
+static struct
+  {
+  int             x264_level;
+  lqt_log_level_t lqt_level;
+  }
+log_levels[] =
+  {
+    { X264_LOG_ERROR,   LQT_LOG_ERROR   },
+    { X264_LOG_WARNING, LQT_LOG_WARNING },
+    { X264_LOG_INFO,    LQT_LOG_INFO    },
+    { X264_LOG_DEBUG,   LQT_LOG_DEBUG   },
+    { X264_LOG_NONE,    LQT_LOG_WARNING }
+  };
+
+static void
+x264_do_log( void * priv, int i_level, const char *psz, va_list argp)
+  {
+  int i;
+  lqt_log_level_t lqt_level;
+  quicktime_t * file;
+  char * msg_string;
+  
+#ifndef HAVE_VASPRINTF
+  int len;
+#endif
+  file = (quicktime_t *)priv;
+  for(i = 0; i < sizeof(log_levels) / sizeof(log_levels[0]); i++)
+    {
+    if(log_levels[i].x264_level == i_level)
+      {
+      lqt_level = log_levels[i].lqt_level;
+      break;
+      }
+    }
+  if(i >= sizeof(log_levels) / sizeof(log_levels[0]))
+    {
+    lqt_log(file, LQT_LOG_WARNING,
+            LOG_DOMAIN, "Invalid log level from x264");
+    return;
+    }
+#ifndef HAVE_VASPRINTF
+  len = vsnprintf((char*)0, 0, psz, argp);
+  msg_string = malloc(len+1);
+  vsnprintf(msg_string, len+1, psz, argp);
+#else
+  vasprintf(&msg_string, psz, argp);
+#endif
+
+  i = strlen(msg_string);
+   
+  if(msg_string[i-1] == '\n')
+    msg_string[i-1] = '\0';
+     
+  
+  lqt_logs(file, lqt_level, LOG_DOMAIN, msg_string);
+  free(msg_string);
+  }
+
+
+static void set_fiel(quicktime_t * file, int track)
+  {
+  quicktime_video_map_t *vtrack = &(file->vtracks[track]);
+  quicktime_stsd_table_t *stsd;
+  
+  stsd = vtrack->track->mdia.minf.stbl.stsd.table;
+
+  if(stsd->has_fiel)
+    return;
+  
+  switch(vtrack->interlace_mode)
+    {
+    case LQT_INTERLACE_NONE:
+      lqt_set_fiel(file, track, 1, 0);
+      break;
+    case LQT_INTERLACE_TOP_FIRST:
+      lqt_set_fiel(file, track, 2, 9);
+      break;
+    case LQT_INTERLACE_BOTTOM_FIRST:
+      lqt_set_fiel(file, track, 2, 14);
+      break;
+    }
+  }
+
 
 static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
   {
@@ -460,6 +559,7 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
   if(!row_pointers)
     {
     vtrack->stream_cmodel = BC_YUV420P;
+    vtrack->chroma_placement = LQT_CHROMA_PLACEMENT_MPEG2;
     return 0;
     }
 
@@ -467,6 +567,7 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 
   if(!codec->initialized)
     {
+
     codec->work_buffer_size = width * height * 3;
     codec->work_buffer = malloc(codec->work_buffer_size); /* Any smaller value here? */
     
@@ -487,13 +588,31 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
     codec->params.i_fps_num = lqt_video_time_scale(file, track);
     codec->params.i_fps_den = lqt_frame_duration(file, track, NULL);
 
+    // Set up logging
+    codec->params.pf_log = x264_do_log;
+    codec->params.p_log_private = file;
+    
+#if X264_BUILD >= 51
+    if(lqt_get_interlace_mode(file, track) != LQT_INTERLACE_NONE)
+      {
+      lqt_log(file, LQT_LOG_INFO, LOG_DOMAIN,
+              "Choosing interlaced encoding");
+      codec->params.b_interlaced = 1;
+      set_fiel(file, track);
+      }
+#endif
+    
     /* Set multipass control */
 
     if(codec->total_passes)
       {
       /* Force ABR */
       codec->params.rc.i_rc_method = X264_RC_ABR;
+#if X264_BUILD < 54
       codec->params.rc.i_rf_constant = 0;
+#else
+      codec->params.rc.f_rf_constant = 0.;
+#endif
       if(codec->pass == 1)
         {
         /* Strings will be made private by x264 */
@@ -507,18 +626,20 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
         codec->params.rc.b_stat_read = 1;
         }
       }
+
+#ifdef DUMP_CONFIG    
+    dump_params(&codec->params);
+#endif
     
     /* Open encoder */
 
+
     codec->enc = x264_encoder_open(&codec->params);
     if(!codec->enc)
       {
-      fprintf(stderr, "x264_encoder_open failed\n");
+      lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "x264_encoder_open failed");
       return result;
       }
-
-    if(codec->params.i_bframe)
-      vtrack->has_b_frames = 1; 
     
     /* Encode global header */
     avcc = create_avcc_atom(codec, &avcc_size);
@@ -528,10 +649,8 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
 
     file->moov.iods.videoProfileId = 0x15;
     
-#ifdef DUMP_CONFIG    
-    dump_params(&codec->params);
-#endif
     codec->initialized = 1;
+
     
     }
 
@@ -633,21 +752,35 @@ enum_t rc_methods[] =
     { "Average bitrate",  X264_RC_ABR },
     { "CRF based VBR",    X264_RC_CRF }
   };
-                                     
+
+#if X264_BUILD >= 63
+enum_t bframe_adaptives[] =
+  {
+    { "None",    X264_B_ADAPT_NONE },
+    { "Fast",    X264_B_ADAPT_FAST },
+    { "Trellis", X264_B_ADAPT_TRELLIS }
+  };
+#endif
 
 static int set_parameter(quicktime_t *file, 
                          int track, 
-                         char *key, 
-                         void *value)
+                         const char *key, 
+                         const void *value)
   {
   int found = 0, i;
   quicktime_x264_codec_t *codec = ((quicktime_codec_t*)file->vtracks[track].codec)->priv;
-
+#if X264_BUILD >= 28
+  INTPARAM("x264_i_threads", codec->params.i_threads);
+#endif
   INTPARAM("x264_i_keyint_max", codec->params.i_keyint_max);
   INTPARAM("x264_i_keyint_min", codec->params.i_keyint_min);
   INTPARAM("x264_i_scenecut_threshold", codec->params.i_scenecut_threshold);
   INTPARAM("x264_i_bframe", codec->params.i_bframe);
+#if X264_BUILD < 63
   INTPARAM("x264_b_bframe_adaptive", codec->params.b_bframe_adaptive);
+#else
+  ENUMPARAM("x264_i_bframe_adaptive", codec->params.i_bframe_adaptive, bframe_adaptives);
+#endif
   INTPARAM("x264_i_bframe_bias", codec->params.i_bframe_bias);
   INTPARAM("x264_b_bframe_pyramid", codec->params.b_bframe_pyramid);
 
@@ -655,7 +788,11 @@ static int set_parameter(quicktime_t *file,
   INTPARAM("x264_i_bitrate", codec->params.rc.i_bitrate);
   
   INTPARAM("x264_i_qp_constant", codec->params.rc.i_qp_constant);
+#if X264_BUILD < 54
   INTPARAM("x264_i_rf_constant", codec->params.rc.i_rf_constant);
+#else
+  FLOATPARAM("x264_f_rf_constant", codec->params.rc.f_rf_constant);
+#endif
   INTPARAM("x264_i_qp_min", codec->params.rc.i_qp_min);
   INTPARAM("x264_i_qp_max", codec->params.rc.i_qp_max);
   INTPARAM("x264_i_qp_step", codec->params.rc.i_qp_step);
@@ -734,12 +871,16 @@ static int set_parameter(quicktime_t *file,
   ENUMPARAM("x264_i_me_method", codec->params.analyse.i_me_method, me_methods);
 
   INTPARAM("x264_i_subpel_refine", codec->params.analyse.i_subpel_refine);
+#if X264_BUILD < 65
   INTPARAM("x264_b_bframe_rdo", codec->params.analyse.b_bframe_rdo);
+#endif
   INTPARAM("x264_i_me_range", codec->params.analyse.i_me_range);
   INTPARAM("x264_i_frame_reference", codec->params.i_frame_reference);
   INTPARAM("x264_b_chroma_me", codec->params.analyse.b_chroma_me);
   INTPARAM("x264_b_mixed_references", codec->params.analyse.b_mixed_references);
+#if X264_BUILD < 65
   INTPARAM("x264_b_bidir_me", codec->params.analyse.b_bidir_me);
+#endif
   INTPARAM("x264_b_weighted_bipred", codec->params.analyse.b_weighted_bipred);
     
   ENUMPARAM("x264_i_direct_mv_pred", codec->params.analyse.i_direct_mv_pred, direct_modes);
@@ -754,7 +895,8 @@ static int set_parameter(quicktime_t *file,
   INTPARAM("x264_i_noise_reduction", codec->params.analyse.i_noise_reduction);
   
   if(!found)
-    fprintf(stderr, "Warning: Unrecognized parameter %s\n", key);
+    lqt_log(file, LQT_LOG_WARNING, LOG_DOMAIN,
+            "Unrecognized parameter %s", key);
   
   return 0;
   }
