@@ -1,11 +1,34 @@
+/*******************************************************************************
+ colr.c
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
 /*
- * $Id: colr.c,v 1.4 2006/04/13 19:25:48 gmerlin Exp $
- *
- * init, read, write handler for the "colr" (Clean Aperture) atom
+
+ Init, read, write handler for the "colr" (Clean Aperture) atom.
+
 */
 
-#include <funcprotos.h>
-#include <quicktime/quicktime.h>
+#include "lqt_private.h"
 #include <string.h>
 
 void quicktime_colr_init(quicktime_colr_t *colr)
@@ -18,11 +41,11 @@ void quicktime_colr_delete(quicktime_colr_t *colr) { }
 void quicktime_colr_dump(quicktime_colr_t *colr)
 {
 
-	printf("     color description (colr)\n");
-	printf("       colorParamType %d\n", colr->colorParamType);
-	printf("       primaries %d\n", colr->primaries);
-	printf("       transferFunction %d\n", colr->transferFunction);
-	printf("       matrix %d\n", colr->matrix);
+	lqt_dump("     color description (colr)\n");
+	lqt_dump("       colorParamType %d\n", colr->colorParamType);
+	lqt_dump("       primaries %d\n", colr->primaries);
+	lqt_dump("       transferFunction %d\n", colr->transferFunction);
+	lqt_dump("       matrix %d\n", colr->matrix);
 }
 
 void quicktime_read_colr(quicktime_t *file, quicktime_colr_t *colr)
