@@ -1,18 +1,44 @@
+/*******************************************************************************
+ lqt_codecfile.c
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ Modified by Napoleon E. Cornejo
+ May 4, 2007, San Salvador, El Salvador
+ - Validated user home directory in create_filename() 
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
 /*
  *   Codec file handling
  */
 
+#include "lqt_private.h"
+#include "lqt_codecinfo_private.h"
+#include <unistd.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <limits.h>
 
-#include <quicktime/lqt.h>
-
-#include <lqt_codecinfo_private.h>
-
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#define LOG_DOMAIN "codecfile"
 
 /*
  *  The keywords are defined globaly, so they are automatically
@@ -64,6 +90,10 @@ static const char * module_filename_key  = "ModuleFilename: ";
 static const char * module_index_key     = "ModuleIndex: ";
 static const char * module_file_time_key = "FileTime: ";
 
+static const char * gettext_domain_key  = "GettextDomain";
+static const char * gettext_directory_key  = "GettextDirectory";
+
+
 /* Types for parameters */
 
 static const char * type_int            = "Integer";
@@ -87,7 +117,15 @@ static const char * real_name_key       = "RealName: ";
 
 static const char * num_options_key     = "NumOptions: ";
 static const char * option_key          = "Options: ";
+static const char * label_key           = "OptionLabels: ";
 
+ /* Encoding colormodels */ 	 
+	  	 
+static const char *
+num_encoding_colormodels_key = "NumEncodingColormodels: "; 	 
+
+static const char *
+encoding_colormodel_key =      "EncodingColormodel: "; 	 
 
 /* Codec order */
 
@@ -104,15 +142,26 @@ static void create_filename()
   {
   /* Obtain the home directory */
 
-  char * home_dir;
+  char * fdir;
   
-  home_dir = getenv("HOME");
-    
-  strcpy(filename_buffer, home_dir);
+  /* First look for a system-wide codec file if available.  If not
+     look into users home */
 
-  strcat(filename_buffer, "/.libquicktime_codecs");
+  fdir = getenv("LQT_CODEC_FILE");
+  if (fdir == NULL ) {
+    
+    lqt_log(NULL, LQT_LOG_DEBUG, LOG_DOMAIN, 
+                    "no system-wide codec file. Looking in user's home.");
+ 
+    fdir = getenv("HOME");
+    strcpy(filename_buffer, fdir);       
+    strcat(filename_buffer, "/.libquicktime_codecs"); 
   
-  }
+  } else 
+     strcpy(filename_buffer, fdir);
+  
+  
+}
 
 static char * __lqt_strdup(const char * string)
   {
@@ -178,6 +227,7 @@ static void read_parameter_info(FILE * input,
   {
   char * pos;
   int options_read = 0;
+  int labels_read = 0;
   
   /* First, get the name */
 
@@ -283,6 +333,8 @@ static void read_parameter_info(FILE * input,
       info->num_stringlist_options = atoi(pos);
       info->stringlist_options = calloc(info->num_stringlist_options,
                                         sizeof(char*));
+      info->stringlist_labels = calloc(info->num_stringlist_options,
+                                       sizeof(char*));
       }
     else if(CHECK_KEYWORD(option_key))
       {
@@ -290,6 +342,12 @@ static void read_parameter_info(FILE * input,
       info->stringlist_options[options_read] = __lqt_strdup(pos);
       options_read++;
       }
+    else if(CHECK_KEYWORD(label_key))
+      {
+      pos = line + strlen(label_key);
+      info->stringlist_labels[labels_read] = __lqt_strdup(pos);
+      labels_read++;
+      }
     else if(CHECK_KEYWORD(help_string_key))
       {
       pos = line + strlen(help_string_key);
@@ -314,6 +372,7 @@ static void read_codec_info(FILE * input, lqt_codec_info_t * codec,
   
   int encoding_parameters_read = 0;
   int decoding_parameters_read = 0;
+  int encoding_colormodels_read = 0;
   
   uint32_t tmp_fourcc;
   
@@ -385,6 +444,22 @@ static void read_codec_info(FILE * input, lqt_codec_info_t * codec,
       pos = line + strlen(module_filename_key);
       codec->module_filename = __lqt_strdup(pos);
       }
+
+    /* Gettext domain */
+    
+    else if(CHECK_KEYWORD(gettext_domain_key))
+      {
+      pos = line + strlen(gettext_domain_key);
+      codec->gettext_domain = __lqt_strdup(pos);
+      }
+
+    /* Gettext directory */
+    
+    else if(CHECK_KEYWORD(gettext_directory_key))
+      {
+      pos = line + strlen(gettext_directory_key);
+      codec->gettext_directory = __lqt_strdup(pos);
+      }
     
     /* Module Index */
     
@@ -456,6 +531,30 @@ static void read_codec_info(FILE * input, lqt_codec_info_t * codec,
         pos = rest;
         }
       }
+
+    /* Number of encoding colormodels */ 	 
+	  	 
+    else if(CHECK_KEYWORD(num_encoding_colormodels_key))
+      {
+      pos = line + strlen(num_encoding_colormodels_key);
+      codec->num_encoding_colormodels = atoi(pos);
+      if(codec->num_encoding_colormodels)
+        codec->encoding_colormodels =
+          malloc(codec->num_encoding_colormodels *
+                 sizeof(int));
+      else 	 
+        codec->encoding_colormodels = (int*)0; 	 
+      } 	 
+
+    /* Encoding colormodels */ 	 
+
+    else if(CHECK_KEYWORD(encoding_colormodel_key)) 	 
+      {
+      pos = line + strlen(encoding_colormodel_key);
+      codec->encoding_colormodels[encoding_colormodels_read] =
+        lqt_string_to_colormodel(pos);
+      encoding_colormodels_read++; 
+      }
     
     /* Number of parameters */
 
@@ -516,16 +615,7 @@ lqt_codec_info_t * lqt_registry_read(char ** audio_order, char ** video_order)
   char * pos = (char*)0;
   lqt_codec_info_t * ret =     (lqt_codec_info_t *)0;
   lqt_codec_info_t * ret_end = (lqt_codec_info_t *)0;
-
-#ifndef NDEBUG
-  int num_audio_codecs = 0;
-  int num_video_codecs = 0;
-#endif
   
-#ifndef NDEBUG
-  //  fprintf(stderr, "Reading codec file %s...", filename_buffer);
-#endif
-
   if(*filename_buffer == '\0')
     create_filename();
   
@@ -534,9 +624,6 @@ lqt_codec_info_t * lqt_registry_read(char ** audio_order, char ** video_order)
   if(!input)
     {
 
-#ifndef NDEBUG
-    //    fprintf(stderr, "failed\n");
-#endif
     return (lqt_codec_info_t*)0;
     }
   
@@ -591,21 +678,10 @@ lqt_codec_info_t * lqt_registry_read(char ** audio_order, char ** video_order)
         }
       read_codec_info(input, ret_end, pos);
 
-#ifndef NDEBUG
-      if(ret_end->type == LQT_CODEC_AUDIO)
-        num_audio_codecs++;
-      else if(ret_end->type == LQT_CODEC_VIDEO)
-        num_video_codecs++;
-#endif
       ret_end->next = (lqt_codec_info_t*)0;
       }
     }
 
-#ifndef NDEBUG
-  //  fprintf(stderr, "done, found %d audio codecs, %d video codecs\n",
-  //          num_audio_codecs, num_video_codecs);
-
-#endif
 
   fclose(input);
   
@@ -700,6 +776,8 @@ static void write_parameter_info(FILE * output,
       
       for(i = 0; i < info->num_stringlist_options; i++)
         fprintf(output, "%s%s\n", option_key, info->stringlist_options[i]);
+      for(i = 0; i < info->num_stringlist_options; i++)
+        fprintf(output, "%s%s\n", label_key, info->stringlist_labels[i]);
       break;
     case LQT_PARAMETER_SECTION:
       break;
@@ -761,7 +839,7 @@ static int write_codec_info(const lqt_codec_info_t * info, FILE * output)
     fprintf(output, "%s%s\n", direction_key, tmp);
 
   fprintf(output, "%s%08x\n", compatibility_key, info->compatibility_flags);
-  
+ 
   
   if(info->num_fourccs)
     {
@@ -798,13 +876,30 @@ static int write_codec_info(const lqt_codec_info_t * info, FILE * output)
     {
     write_parameter_info(output, &(info->decoding_parameters[i]), 0);
     }
- 
+  
+  if((info->type == LQT_CODEC_VIDEO) &&
+     (info->direction != LQT_DIRECTION_DECODE))
+    {
+    fprintf(output, "%s%d\n", num_encoding_colormodels_key,
+            info->num_encoding_colormodels);
+    
+    for(i = 0; i < info->num_encoding_colormodels; i++)
+      {
+      fprintf(output, "%s%s\n", encoding_colormodel_key,
+              lqt_colormodel_to_string(info->encoding_colormodels[i]));
+      }
+    }
   
   /* Module filename and index */
   fprintf(output, "%s%s\n", module_filename_key, info->module_filename);
   fprintf(output, "%s%d\n", module_index_key, info->module_index);
   fprintf(output, "%s%u\n", module_file_time_key, info->file_time);
-    
+
+  if(info->gettext_domain)
+    fprintf(output, "%s%s\n", gettext_domain_key, info->gettext_domain);
+  if(info->gettext_directory)
+    fprintf(output, "%s%s\n", gettext_directory_key, info->gettext_directory);
+  
   if(fprintf(output, "%s\n", end_codec_key) < 0)
     return 0;
   return 1;
@@ -822,9 +917,6 @@ void lqt_registry_write()
   if(*filename_buffer == '\0')
     create_filename();
 
-#ifndef NDEBUG
-  //  fprintf(stderr, "Writing codec file %s...", filename_buffer);
-#endif
   
   output = fopen(filename_buffer, "w");
   
@@ -840,7 +932,7 @@ void lqt_registry_write()
   
   fprintf(output, "# This is the codec database file for libquicktime\n\
 # It is automatically generated and should not be edited.\n\
-# If you canged it, and your libquicktime program doesn't work\n\
+# If you changed it and your libquicktime program doesn't work\n\
 # anymore, delete it, and you will get a new one\n");
 
   /* Write the sort strings */
@@ -849,10 +941,10 @@ void lqt_registry_write()
     {
     codec_info = lqt_audio_codecs;
     
-    fprintf(output, audio_order_key);
+    fprintf(output, "%s", audio_order_key);
     for(i = 0; i < lqt_num_audio_codecs; i++)
       {
-      fprintf(output, codec_info->name);
+      fprintf(output, "%s", codec_info->name);
       if(i == lqt_num_audio_codecs - 1)
         fprintf(output, "\n");
       else
@@ -864,10 +956,10 @@ void lqt_registry_write()
   if(lqt_num_video_codecs)
     {
     codec_info = lqt_video_codecs;
-    fprintf(output, video_order_key);
+    fprintf(output, "%s", video_order_key);
     for(i = 0; i < lqt_num_video_codecs; i++)
       {
-      fprintf(output, codec_info->name);
+      fprintf(output, "%s", codec_info->name);
       if(i == lqt_num_video_codecs - 1)
         fprintf(output, "\n");
       else
@@ -895,15 +987,12 @@ void lqt_registry_write()
   fclose(output);
   lqt_registry_unlock();
 
-#ifndef NDEBUG
-  //  fprintf(stderr, "done\n");
-#endif
   return;
 fail:
   fclose(output);
   lqt_registry_unlock();
-#ifndef NDEBUG
-  fprintf(stderr, "%s could not be written, deleting imcomplete file\n", filename_buffer);
-#endif
+  
+  lqt_log(NULL, LQT_LOG_INFO, LOG_DOMAIN,
+          "%s could not be written, deleting imcomplete file", filename_buffer);
   remove(filename_buffer);
   }
