@@ -1,29 +1,43 @@
-#include <string.h>
-#include <limits.h>
-#include <dlfcn.h>
+/*******************************************************************************
+ lqt_codecinfo.c
 
-#include <pthread.h>
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
 
-#include <sys/types.h>
-#include <dirent.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <quicktime/lqt.h>
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
 
-#include <lqt_codecinfo_private.h>
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
 
-/*
- *  Define LQT_LIBQUICKTIME to prevent compiling the
- *  get_codec_api_version() function
- */
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
 
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
+#include "lqt_codecinfo_private.h"
 #define LQT_LIBQUICKTIME
 #include <quicktime/lqt_codecapi.h>
+#include <sys/stat.h>
+#include <pthread.h>
+#include <dlfcn.h>
+#include <dirent.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <string.h>
+#include <limits.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <libintl.h>
+
+#define LOG_DOMAIN "codecinfo"
 
 /* Public function (lqt.h) */
 
@@ -79,20 +93,9 @@ void lqt_registry_unlock()
   pthread_mutex_unlock(&codecs_mutex);
   }
 
-int lqt_registry_initialized()
-  {
-  int ret;
-  
-  lqt_registry_lock();
-  ret = registry_init_done;
-  lqt_registry_unlock();
-  
-  return ret;
-  }
-
 /* Free memory of parameter info */
 
-void destroy_parameter_info(lqt_parameter_info_t * p)
+static void destroy_parameter_info(lqt_parameter_info_t * p)
   {
   int i;
   if(p->name)
@@ -117,6 +120,12 @@ void destroy_parameter_info(lqt_parameter_info_t * p)
           free(p->stringlist_options[i]);
         free(p->stringlist_options);
         }
+      if(p->stringlist_labels)
+        {
+        for(i = 0; i < p->num_stringlist_options; i++)
+          free(p->stringlist_labels[i]);
+        free(p->stringlist_labels);
+        }
       break;
     default: /* Keep gcc quiet */
       break;
@@ -125,7 +134,7 @@ void destroy_parameter_info(lqt_parameter_info_t * p)
 
 /* Free memory of codec info (public) */
 
-void destroy_codec_info(lqt_codec_info_t * ptr)
+static void destroy_codec_info(lqt_codec_info_t * ptr)
   {
   int i;
 
@@ -148,6 +157,12 @@ void destroy_codec_info(lqt_codec_info_t * ptr)
   if(ptr->module_filename)        /* Module filename       */
     free(ptr->module_filename);
 
+  if(ptr->gettext_domain)
+    free(ptr->gettext_domain);
+  if(ptr->gettext_directory)
+    free(ptr->gettext_directory);
+  if(ptr->encoding_colormodels)
+    free(ptr->encoding_colormodels);
   
   if(ptr->encoding_parameters)
     {
@@ -206,7 +221,7 @@ copy_parameter_info(lqt_parameter_info_t * ret,
     ret->real_name = __lqt_strdup(info->real_name);
   if(info->help_string)
     ret->help_string = __lqt_strdup(info->help_string);
-
+  
   ret->type = info->type;
 
   switch(ret->type)
@@ -226,10 +241,16 @@ copy_parameter_info(lqt_parameter_info_t * ret,
       ret->num_stringlist_options = info->num_stringlist_options;
       ret->stringlist_options = calloc(ret->num_stringlist_options,
                                        sizeof(char*));
-
+      ret->stringlist_labels = calloc(ret->num_stringlist_options,
+                                      sizeof(char*));
+      
       for(i = 0; i < ret->num_stringlist_options; i++)
+        {
         ret->stringlist_options[i] =
           __lqt_strdup(info->stringlist_options[i]);
+        ret->stringlist_labels[i] =
+          __lqt_strdup(info->stringlist_labels[i]);
+        }
 
       break;
     case LQT_PARAMETER_SECTION: /* String with options */
@@ -251,7 +272,7 @@ copy_codec_info(const lqt_codec_info_t * info)
   {
   int i;
   lqt_codec_info_t * ret = calloc(1, sizeof(lqt_codec_info_t));
-
+  
   ret->compatibility_flags = info->compatibility_flags;
     
   if(info->name)
@@ -260,7 +281,17 @@ copy_codec_info(const lqt_codec_info_t * info)
     ret->long_name = __lqt_strdup(info->long_name);
   if(info->description)
     ret->description = __lqt_strdup(info->description);
+  
+  if(info->gettext_domain)
+    ret->gettext_domain = __lqt_strdup(info->gettext_domain);
+  else
+    ret->gettext_domain = __lqt_strdup(PACKAGE);
 
+  if(info->gettext_directory)
+    ret->gettext_directory = __lqt_strdup(info->gettext_directory);
+  else
+    ret->gettext_directory = __lqt_strdup(LOCALE_DIR);
+  
   if(info->module_filename)
     ret->module_filename = __lqt_strdup(info->module_filename);
 
@@ -277,6 +308,15 @@ copy_codec_info(const lqt_codec_info_t * info)
       ret->fourccs[i] = __lqt_fourccdup(info->fourccs[i]);
     }
 
+  ret->num_encoding_colormodels = info->num_encoding_colormodels;
+  if(ret->num_encoding_colormodels)
+    {
+    ret->encoding_colormodels = malloc(ret->num_encoding_colormodels *
+                                       sizeof(*ret->encoding_colormodels));
+    for(i = 0; i < ret->num_encoding_colormodels; i++)
+      ret->encoding_colormodels[i] = info->encoding_colormodels[i];
+    }
+  
   ret->num_wav_ids = info->num_wav_ids;
   if(ret->num_wav_ids)
     {
@@ -292,7 +332,7 @@ copy_codec_info(const lqt_codec_info_t * info)
   if(ret->num_encoding_parameters)
     {
     ret->encoding_parameters =
-      calloc(ret->num_encoding_parameters, sizeof(lqt_parameter_info_t));
+      calloc(ret->num_encoding_parameters+1, sizeof(lqt_parameter_info_t));
 
     for(i = 0; i < ret->num_encoding_parameters; i++)
       copy_parameter_info(&(ret->encoding_parameters[i]),
@@ -411,25 +451,15 @@ static lqt_codec_info_t * load_codec_info_from_plugin(char * plugin_filename,
   
   lqt_codec_info_t * ret = (lqt_codec_info_t*)0;
 
-#ifndef NDEBUG  
-  //  fprintf(stderr, "Trying to load %s...", plugin_filename);
-#endif
   
   module = dlopen(plugin_filename, RTLD_NOW);
   if(!module)
     {
-#ifndef NDEBUG  
-    //    fprintf(stderr, "\n");
-#endif
-    fprintf(stderr, "load_codec_info_from_plugin: dlopen failed for %s: %s\n",
-             plugin_filename, dlerror());
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "dlopen failed for %s: %s",
+            plugin_filename, dlerror());
     return ret;
     }
-#ifndef NDEBUG  
-  //  else
-  //    fprintf(stderr, "success\n");
-#endif
-
+  
   /* Now, get the codec parameters */
 
   /* Check the api version */
@@ -438,7 +468,8 @@ static lqt_codec_info_t * load_codec_info_from_plugin(char * plugin_filename,
 
   if(!get_codec_api_version)
     {
-    fprintf(stderr, "module %s has no API version and is thus terribly old\n",
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN,
+            "Module %s has no API version and is thus terribly old",
             plugin_filename);
     dlclose(module);
     return ret;
@@ -448,9 +479,8 @@ static lqt_codec_info_t * load_codec_info_from_plugin(char * plugin_filename,
 
   if(codec_api_version_module != codec_api_version_us)
     {
-    fprintf(stderr, "Codec interface version mismatch of module %s\n\
-Module interface version       %d\n\
-Libquicktime interface version %d\n", 
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN,
+            "Codec interface version mismatch of module %s: %d [module] != %d [lqt]",
             plugin_filename,
             codec_api_version_module,
             codec_api_version_us);
@@ -461,8 +491,8 @@ Libquicktime interface version %d\n",
 
   if(!get_num_codecs)
     {
-    fprintf(stderr, "Symbol %s not found in %s\n",
-            "get_num_codecs", plugin_filename);
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "Symbol get_num_codecs not found in %s",
+            plugin_filename);
     dlclose(module);
     return ret;
     }
@@ -470,8 +500,8 @@ Libquicktime interface version %d\n",
   get_codec_info = (lqt_codec_info_static_t*(*)(int))(dlsym(module, "get_codec_info"));
   if(!get_codec_info)
     {
-    fprintf(stderr, "Symbol %s not found in %s\n",
-            "get_codec_info", plugin_filename);
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "Symbol get_codec_info not found in %s",
+            plugin_filename);
     dlclose(module);
     return ret;
     }
@@ -481,7 +511,8 @@ Libquicktime interface version %d\n",
   num_codecs = get_num_codecs();
   if(!num_codecs)
     {
-    fprintf(stderr, "No codecs found\n");
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "No codecs found in %s",
+            plugin_filename);
     dlclose(module);
     return ret;
     }
@@ -526,9 +557,6 @@ static void register_codecs(lqt_codec_info_t * list,
     {
     if(list->type == LQT_CODEC_AUDIO)
       {
-#ifndef NDEBUG
-      //      fprintf(stderr, "Registering audio codec %s\n", list->name);
-#endif
       if(*audio_codecs_end)
         {
         (*audio_codecs_end)->next = list;
@@ -543,9 +571,6 @@ static void register_codecs(lqt_codec_info_t * list,
       }
     if(list->type == LQT_CODEC_VIDEO)
       {
-#ifndef NDEBUG
-      //      fprintf(stderr, "Registering video codec %s\n", list->name);
-#endif
       if((*video_codecs_end))
         {
         (*video_codecs_end)->next = list;
@@ -582,9 +607,6 @@ static int scan_for_plugins(const char * plugin_dir, lqt_codec_info_t ** databas
   
   /* Set the end pointers so we can quickly add codecs after */
 
-#ifndef NDEBUG
-  //  fprintf(stderr, "Scanning %s for plugins\n", plugin_dir);
-#endif
   
   audio_codecs_end = lqt_audio_codecs;
 
@@ -602,9 +624,8 @@ static int scan_for_plugins(const char * plugin_dir, lqt_codec_info_t ** databas
 
   if(!directory)
     {
-    fprintf(stderr, "Cannot open plugin directory %s\n\
-Did you forget \"make install\"? You need it because\n\
-libquicktime cannot load plugins out of the sourcetree\n", plugin_dir);
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN,
+            "Cannot open plugin directory %s (forgot make install?)", plugin_dir);
     return 0;
     }
 
@@ -628,7 +649,7 @@ libquicktime cannot load plugins out of the sourcetree\n", plugin_dir);
     if(!pos)
       continue;
     
-    if(strcmp(pos, ".so"))
+    if(strcmp(pos, MODULE_EXT))
       continue;
     
     /* Now, the file should be a valid plugin, construct the filename */
@@ -641,28 +662,17 @@ libquicktime cannot load plugins out of the sourcetree\n", plugin_dir);
     if(!S_ISREG(status.st_mode))
       continue;
 
-#ifndef NDEBUG
-    //    fprintf(stderr, "Found %s...", directory_entry->d_name);
-#endif
     
     codecs = find_codec_by_filename(database, filename, status.st_ctime);
     
     if(codecs) /* Codec information found in database */
       {
-#ifndef NDEBUG
-      //      fprintf(stderr,
-      //              "Already in database, registering all codecs\n");
-#endif
       register_codecs(codecs,
                       &audio_codecs_end,
                       &video_codecs_end);
       }
     else /* Load the informations from the module */
       {
-#ifndef NDEBUG
-      //      fprintf(stderr,
-      //              "Getting codec info from module\n");
-#endif
       codecs = load_codec_info_from_plugin(filename, status.st_ctime);
       register_codecs(codecs,
                       &audio_codecs_end,
@@ -709,16 +719,21 @@ void lqt_registry_init()
   lqt_codec_info_t * tmp_file_codecs;
   const char* plugin_dir = PLUGIN_DIR;
 
+  lqt_registry_lock();
+  if(registry_init_done)
+    {
+    lqt_registry_unlock();
+    return;
+    }
+
+  registry_init_done = 1;
+  
   /* Check for environment variable for plugin dir */
   if(getenv("LIBQUICKTIME_PLUGIN_DIR"))
     {
     plugin_dir = getenv("LIBQUICKTIME_PLUGIN_DIR");
     }
   
-  lqt_registry_lock();
-  
-  registry_init_done = 1;
-  
   if(lqt_audio_codecs || lqt_video_codecs)
     {
     lqt_registry_unlock();
@@ -739,9 +754,6 @@ void lqt_registry_init()
   
   while(file_codecs)
     {
-#ifndef NDEBUG
-    //    fprintf(stderr, "Removing codec %s from registry\n", file_codecs->name);
-#endif
     tmp_file_codecs = file_codecs;
     file_codecs = file_codecs->next;
     destroy_codec_info(tmp_file_codecs);
@@ -821,48 +833,6 @@ const lqt_codec_info_t * lqt_get_video_codec_info(int index)
 
 /* Thread save methods of getting codec infos */
 
-lqt_codec_info_t * lqt_get_audio_codec_info_c(int index)
-  {
-  const lqt_codec_info_t * info;
-  lqt_codec_info_t * ret;
-
-  if (!lqt_registry_initialized()) /* also init registry */
-  	lqt_registry_init();
-	
-  lqt_registry_lock();
-
-  info = lqt_get_audio_codec_info(index);
-
-  if(info)
-    ret = copy_codec_info(info);
-  else
-    ret = (lqt_codec_info_t*)0;
-  lqt_registry_unlock();
-  return ret;
-  }
-
-lqt_codec_info_t * lqt_get_video_codec_info_c(int index)
-  {
-  const lqt_codec_info_t * info;
-  lqt_codec_info_t * ret;
-
-  if (!lqt_registry_initialized()) /* also init registry */
-  	lqt_registry_init();
-
-  lqt_registry_lock();
-
-  info = lqt_get_video_codec_info(index);
-
-  if(info)
-    ret = copy_codec_info(info);
-  else
-    ret = (lqt_codec_info_t*)0;
-  lqt_registry_unlock();
-  return ret;
-  }
-
-
-
 static void 
 create_parameter_info(lqt_parameter_info_t * ret,
                     const lqt_parameter_info_static_t * info)
@@ -896,7 +866,7 @@ create_parameter_info(lqt_parameter_info_t * ret,
       ret->val_default.val_string = __lqt_strdup(info->val_default.val_string);
       if(!info->stringlist_options)
         {
-        fprintf(stderr, "Stringlist parameter %s has NULL options\n",
+        lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "Stringlist parameter %s has NULL options",
                 info->name);
         return;
         }
@@ -919,7 +889,26 @@ create_parameter_info(lqt_parameter_info_t * ret,
       for(i = 0; i < ret->num_stringlist_options; i++)
         {
           ret->stringlist_options[i] =
-          __lqt_strdup(info->stringlist_options[i]);
+            __lqt_strdup(info->stringlist_options[i]);
+        }
+
+      /* Labels */
+      ret->stringlist_labels = malloc(ret->num_stringlist_options * sizeof(char *));
+      if(info->stringlist_labels)
+        {
+        for(i = 0; i < ret->num_stringlist_options; i++)
+          {
+          ret->stringlist_labels[i] =
+            __lqt_strdup(info->stringlist_labels[i]);
+          }
+        }
+      else
+        {
+        for(i = 0; i < ret->num_stringlist_options; i++)
+          {
+          ret->stringlist_labels[i] =
+            __lqt_strdup(info->stringlist_options[i]);
+          }
         }
       break;
     default:
@@ -936,7 +925,7 @@ lqt_create_codec_info(const lqt_codec_info_static_t * template)
 
   if(!template->fourccs)
     {
-    fprintf(stderr, "Codec %s has no fourccs defined\n", template->name);
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "Codec %s has no fourccs defined", template->name);
     return (lqt_codec_info_t*)0;
     }
   
@@ -948,6 +937,11 @@ lqt_create_codec_info(const lqt_codec_info_static_t * template)
   ret->long_name =   __lqt_strdup(template->long_name);
   ret->description = __lqt_strdup(template->description);
 
+  if(template->gettext_domain)
+    ret->gettext_domain = __lqt_strdup(template->gettext_domain);
+  if(template->gettext_directory)
+    ret->gettext_directory = __lqt_strdup(template->gettext_directory);
+  
   ret->type      = template->type;
   ret->direction = template->direction;
 
@@ -967,6 +961,26 @@ lqt_create_codec_info(const lqt_codec_info_static_t * template)
   for(i = 0; i < ret->num_fourccs; i++)
     ret->fourccs[i] = __lqt_fourccdup(template->fourccs[i]);
 
+  /* Copy supported encoding colormodels */
+
+  ret->num_encoding_colormodels = 0;
+
+  if(template->encoding_colormodels)
+    {
+    while(1)
+      {
+      if(template->encoding_colormodels[ret->num_encoding_colormodels] != LQT_COLORMODEL_NONE)
+        ret->num_encoding_colormodels++;
+      else
+        break;
+      }
+
+    ret->encoding_colormodels =
+      malloc(ret->num_encoding_colormodels * sizeof(*ret->encoding_colormodels));
+    for(i = 0; i < ret->num_encoding_colormodels; i++)
+      ret->encoding_colormodels[i] = template->encoding_colormodels[i];
+    }
+  
   /* Copy wav_ids */
 
   ret->num_wav_ids = 0;
@@ -1052,81 +1066,81 @@ lqt_create_codec_info(const lqt_codec_info_static_t * template)
 static void dump_codec_parameter(lqt_parameter_info_t * p)
   {
   int i;
-  fprintf(stderr, "Parameter: %s (%s) ", p->name,
+  lqt_dump("Parameter: %s (%s) ", p->name,
           p->real_name);
-  fprintf(stderr, "Type: ");
+  lqt_dump("Type: ");
   switch(p->type)
     {
     case LQT_PARAMETER_INT:
-      fprintf(stderr, "Integer, Default Value: %d ",
+      lqt_dump("Integer, Default Value: %d ",
               p->val_default.val_int);
 
       if(p->val_min.val_int < p->val_max.val_int)
-        fprintf(stderr, "(%d..%d)\n",
+        lqt_dump("(%d..%d)\n",
                 p->val_min.val_int, p->val_max.val_int);
       else
-        fprintf(stderr, "(unlimited)\n");
+        lqt_dump("(unlimited)\n");
       break;
     case LQT_PARAMETER_FLOAT:
-      fprintf(stderr, "Float, Default Value: %f ",
+      lqt_dump("Float, Default Value: %f ",
               p->val_default.val_float);
 
       if(p->val_min.val_float < p->val_max.val_float)
-        fprintf(stderr, "(%f..%f)\n",
+        lqt_dump("(%f..%f)\n",
                 p->val_min.val_float, p->val_max.val_float);
       else
-        fprintf(stderr, "(unlimited)\n");
+        lqt_dump("(unlimited)\n");
       break;
     case LQT_PARAMETER_STRING:
-      fprintf(stderr, "String, Default Value : %s\n",
+      lqt_dump("String, Default Value : %s\n",
               (p->val_default.val_string ? p->val_default.val_string : "NULL"));
       break;
     case LQT_PARAMETER_STRINGLIST:
-      fprintf(stderr, "Stringlist, Default Value : %s\n",
+      lqt_dump("Stringlist, Default Value : %s\n",
               (p->val_default.val_string ? p->val_default.val_string :
                "NULL"));
-      fprintf(stderr, "Options: ");
+      lqt_dump("Options: ");
       for(i = 0; i < p->num_stringlist_options; i++)
-        fprintf(stderr, "%s ", p->stringlist_options[i]);
-      fprintf(stderr, "\n");
+        lqt_dump("%s ", p->stringlist_options[i]);
+      lqt_dump("\n");
       break;
     case LQT_PARAMETER_SECTION:
-      fprintf(stderr, "Section");
+      lqt_dump("Section");
     }
   if(p->help_string)
-    fprintf(stderr, "Help string: %s\n", p->help_string);
+    lqt_dump("Help string: %s\n", p->help_string);
   }
 
 void lqt_dump_codec_info(const lqt_codec_info_t * info)
   {
   int i;
-  fprintf(stderr, "Codec: %s (%s)\n", info->long_name, info->name);
+  lqt_dump("Codec: %s (%s)\n", info->long_name, info->name);
   
-  fprintf(stderr, "Type: %s Direction: ",
+  lqt_dump("Type: %s Direction: ",
           (info->type == LQT_CODEC_AUDIO ? "Audio, " : "Video, ") );
   switch(info->direction)
     {
     case LQT_DIRECTION_ENCODE:
-      fprintf(stderr, "Encode\n");
+      lqt_dump("Encode\n");
       break;
     case LQT_DIRECTION_DECODE:
-      fprintf(stderr, "Decode\n");
+      lqt_dump("Decode\n");
       break;
     case LQT_DIRECTION_BOTH:
-      fprintf(stderr, "Encode/Decode\n");
+      lqt_dump("Encode/Decode\n");
       break;
     }
 
-  fprintf(stderr, "Description:\n%s\n", info->description);
+  lqt_dump("Description:\n%s\n", info->description);
 
-  fprintf(stderr, "Four character codes: (fourccs)\n");
+  lqt_dump("Four character codes: (fourccs)\n");
   for(i = 0; i < info->num_fourccs; i++)
-    fprintf(stderr, "%s (0x%08x)\n", info->fourccs[i],
+    lqt_dump("%s (0x%08x)\n", info->fourccs[i],
             LQT_STRING_2_FOURCC(info->fourccs[i]));
 
   if(!info->num_encoding_parameters)
     {
-    fprintf(stderr, "No settable parameters for encoding\n");
+    lqt_dump("No settable parameters for encoding\n");
     }
   else
     {
@@ -1136,14 +1150,14 @@ void lqt_dump_codec_info(const lqt_codec_info_t * info)
 
   if(!info->num_encoding_parameters)
     {
-    fprintf(stderr, "No settable parameters for decoding\n");
+    lqt_dump("No settable parameters for decoding\n");
     }
   else
     {
     for(i = 0; i < info->num_decoding_parameters; i++)
       dump_codec_parameter(&(info->decoding_parameters[i]));
     }
-  fprintf(stderr, "Module filename: %s\nIndex inside module: %d\n",
+  lqt_dump("Module filename: %s\nIndex inside module: %d\n",
           info->module_filename, info->module_index);
 
   }
@@ -1167,8 +1181,8 @@ lqt_codec_info_t ** lqt_find_audio_codec(char * fourcc, int encode)
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
 
-  if (!lqt_registry_initialized()) /* also init registry */
-    lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
     
   lqt_registry_lock();
   
@@ -1180,8 +1194,8 @@ lqt_codec_info_t ** lqt_find_audio_codec(char * fourcc, int encode)
       {
       if(MATCH_FOURCC(ptr->fourccs[j], fourcc))
         {
-        if(((encode) & (ptr->direction != LQT_DIRECTION_DECODE)) ||
-           (ptr->direction != LQT_DIRECTION_ENCODE))
+        if((encode && (ptr->direction != LQT_DIRECTION_DECODE)) ||
+           (!encode && (ptr->direction != LQT_DIRECTION_ENCODE)))
           {
           tmp_ptr = ptr;
           break;
@@ -1209,8 +1223,8 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_wav_id(int wav_id, int encode)
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
 
-  if (!lqt_registry_initialized()) /* also init registry */
-  	lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
   
   lqt_registry_lock();
   
@@ -1222,8 +1236,8 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_wav_id(int wav_id, int encode)
       {
       if(ptr->wav_ids[j] == wav_id)
         {
-        if(((encode) & (ptr->direction != LQT_DIRECTION_DECODE)) ||
-           (ptr->direction != LQT_DIRECTION_ENCODE))
+        if((encode && (ptr->direction != LQT_DIRECTION_DECODE)) ||
+           (!encode && (ptr->direction != LQT_DIRECTION_ENCODE)))
           {
           tmp_ptr = ptr;
           break;
@@ -1252,8 +1266,8 @@ lqt_codec_info_t ** lqt_find_video_codec(char * fourcc, int encode)
 
   lqt_codec_info_t ** ret = (lqt_codec_info_t **)0;
   
-  if (!lqt_registry_initialized()) /* also init registry */
-     lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
   
   lqt_registry_lock();
   
@@ -1265,8 +1279,8 @@ lqt_codec_info_t ** lqt_find_video_codec(char * fourcc, int encode)
       {
       if(MATCH_FOURCC(ptr->fourccs[j], fourcc))
         {
-        if(((encode) & (ptr->direction != LQT_DIRECTION_DECODE)) ||
-           (ptr->direction != LQT_DIRECTION_ENCODE))
+        if((encode && (ptr->direction != LQT_DIRECTION_DECODE)) ||
+           (!encode && (ptr->direction != LQT_DIRECTION_ENCODE)))
           {
           tmp_ptr = ptr;
           break;
@@ -1298,8 +1312,8 @@ lqt_codec_info_t ** lqt_query_registry(int audio, int video,
   const lqt_codec_info_t * info;
   int num_codecs = 0, num_added = 0, i;
 
-  if (!lqt_registry_initialized()) /* also init registry */
-    lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
 
   lqt_registry_lock();
 
@@ -1369,8 +1383,8 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_name(const char * name)
   if(!name)
     return ret;
   
-  if (!lqt_registry_initialized()) /* also init registry */
-     lqt_registry_init();
+   /* also init registry */
+  lqt_registry_init();
 
   lqt_registry_lock();
 
@@ -1389,14 +1403,6 @@ lqt_codec_info_t ** lqt_find_audio_codec_by_name(const char * name)
     }
   lqt_registry_unlock();
 
-#ifndef NDEBUG
-  if(ret)
-    fprintf(stderr, "lqt_find_audio_codec_by_name(%s) success\n",
-            name);
-  else
-    fprintf(stderr, "lqt_find_audio_codec_by_name(%s) failed\n",
-            name);
-#endif
 
   return ret;
   }
@@ -1410,8 +1416,8 @@ lqt_codec_info_t ** lqt_find_video_codec_by_name(const char * name)
   if(!name)
     return ret;
   
-  if (!lqt_registry_initialized()) /* also init registry */
-     lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
   
   lqt_registry_lock();
 
@@ -1430,20 +1436,12 @@ lqt_codec_info_t ** lqt_find_video_codec_by_name(const char * name)
     }
   lqt_registry_unlock();
 
-#ifndef NDEBUG
-  if(ret)
-    fprintf(stderr, "lqt_find_video_codec_by_name(%s) success\n",
-            name);
-  else
-    fprintf(stderr, "lqt_find_video_codec_by_name(%s) failed\n",
-            name);
-#endif
 
   return ret;
   }
 
 /*
- *  Get infos about the Codecs of a file
+ *  Get infos about the codecs of a file
  *  To be called after quicktime_open() when reading
  *  or quicktime_set_audio()/quicktime_set_video() when writing
  */
@@ -1490,8 +1488,8 @@ void lqt_set_default_parameter(lqt_codec_type type, int encode,
   lqt_codec_info_t * codec_info;
   lqt_parameter_info_t * parameter_info;
   
-  if (!lqt_registry_initialized()) /* also init registry */
-     lqt_registry_init();
+  /* also init registry */
+  lqt_registry_init();
 
   lqt_registry_lock();
 
@@ -1511,7 +1509,7 @@ void lqt_set_default_parameter(lqt_codec_type type, int encode,
 
   if(!codec_info)
     {
-    fprintf(stderr, "lqt_set_default_parameter: No %s codec %s found\n",
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "lqt_set_default_parameter: No %s codec %s found",
             ((type == LQT_CODEC_AUDIO) ? "audio" : "video"),  codec_name);
     lqt_registry_unlock();
     return;
@@ -1541,8 +1539,8 @@ void lqt_set_default_parameter(lqt_codec_type type, int encode,
 
   if(!parameter_found)
     {
-    fprintf(stderr, "lqt_set_default_parameter: No parameter %s for codec \
-%s found\n",
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN,
+            "lqt_set_default_parameter: No parameter %s for codec %s found",
             parameter_name,
             codec_name);
     lqt_registry_unlock();
@@ -1555,23 +1553,9 @@ void lqt_set_default_parameter(lqt_codec_type type, int encode,
     {
     case LQT_PARAMETER_INT:
       parameter_info[i].val_default.val_int = val->val_int;
-#ifndef NDEBUG
-  fprintf(stderr,
-          "%s parameter %s for codec %s (value: %d) stored in registry\n",
-          (encode ? "Encoding" : "Decoding"),
-          parameter_name,
-          codec_name, parameter_info[i].val_default.val_int);
-#endif
       break;
     case LQT_PARAMETER_FLOAT:
       parameter_info[i].val_default.val_float = val->val_float;
-#ifndef NDEBUG
-  fprintf(stderr,
-          "%s parameter %s for codec %s (value: %f) stored in registry\n",
-          (encode ? "Encoding" : "Decoding"),
-          parameter_name,
-          codec_name, parameter_info[i].val_default.val_float);
-#endif
       break;
     case LQT_PARAMETER_STRING:
     case LQT_PARAMETER_STRINGLIST:
@@ -1579,13 +1563,6 @@ void lqt_set_default_parameter(lqt_codec_type type, int encode,
         free(parameter_info[i].val_default.val_string);
       parameter_info[i].val_default.val_string =
         __lqt_strdup(val->val_string);
-#ifndef NDEBUG
-  fprintf(stderr,
-          "%s parameter %s for codec %s (value: \"%s\") stored in registry\n",
-          (encode ? "Encoding" : "Decoding"),
-          parameter_name,
-          codec_name, parameter_info[i].val_default.val_string);
-#endif
       break;
     case LQT_PARAMETER_SECTION:
       break;
@@ -1621,7 +1598,8 @@ void lqt_restore_default_parameters(lqt_codec_info_t * codec_info,
   module = dlopen(codec_info->module_filename, RTLD_NOW);
   if(!module)
     {
-    fprintf(stderr, "lqt_restore_default_parameters: dlopen failed for %s: %s\n",
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN,
+            "lqt_restore_default_parameters: dlopen failed for %s: %s",
              codec_info->module_filename, dlerror());
     return;
     }
@@ -1629,7 +1607,7 @@ void lqt_restore_default_parameters(lqt_codec_info_t * codec_info,
   get_codec_info = (lqt_codec_info_static_t*(*)(int))(dlsym(module, "get_codec_info"));
   if(!get_codec_info)
     {
-    fprintf(stderr, "Symbol %s not found in %s\n",
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "Symbol %s not found in %s",
             "get_codec_info", codec_info->module_filename);
     return;
     }
@@ -1638,7 +1616,7 @@ void lqt_restore_default_parameters(lqt_codec_info_t * codec_info,
   
   if(!info_from_module)
     {
-    fprintf(stderr, "Couldn't get codec info for %s from_module %s\n",
+    lqt_log(NULL, LQT_LOG_ERROR, LOG_DOMAIN, "Couldn't get codec info for %s from_module %s",
             codec_info->long_name, codec_info->module_filename);
     return;
     }
@@ -1658,10 +1636,6 @@ void lqt_restore_default_parameters(lqt_codec_info_t * codec_info,
         copy_parameter_value(&(codec_info->encoding_parameters[i].val_default),
                              &(info_from_module->encoding_parameters[i].val_default),
                              codec_info->encoding_parameters[i].type);
-#ifndef NDEBUG
-        //        fprintf(stderr, "Setting encoding parameter %s\n",
-        //                codec_info->encoding_parameters[i].name);
-#endif
 
         }
       }
@@ -1680,10 +1654,6 @@ void lqt_restore_default_parameters(lqt_codec_info_t * codec_info,
         copy_parameter_value(&(codec_info->decoding_parameters[i].val_default),
                              &(info_from_module->decoding_parameters[i].val_default),
                              codec_info->decoding_parameters[i].type);
-#ifndef NDEBUG
-        //        fprintf(stderr, "Setting decoding parameter %s\n",
-        //                codec_info->decoding_parameters[i].name);
-#endif
 
         }
       }
@@ -1726,9 +1696,6 @@ sort_codecs_internal(lqt_codec_info_t * original, char * names)
   if(!end_pos)
     end_pos = pos + strlen(pos);
 
-#ifndef NDEBUG
-  //  fprintf(stderr, "Sorting codecs, sort string: %s\n", pos);
-#endif
   
   while(1)
     {
@@ -1849,7 +1816,6 @@ static void __lqt_cleanup_codecinfo() __attribute__ ((destructor));
 
 static void __lqt_cleanup_codecinfo()
   {
-  //  fprintf(stderr, "Deleting quicktime codecs\n");
   lqt_registry_destroy();
   }
 
